/*
 * Security Ninja PRO
 * Main backend JS
 * (c) Web factory Ltd, 2012 - 2018
 */

function sn_block_ui(content_el) {
  jQuery('html.wp-toolbar').addClass('sn-overlay-active');
  jQuery('#wpadminbar').addClass('sn-overlay-active');
  jQuery('#sn_overlay .wf-sn-overlay-outer').css('height', (jQuery(window).height() - 200) + 'px');
  jQuery('#sn_overlay').show();

  if (content_el) {
    jQuery(content_el, '#sn_overlay').show();
  }
} // sn_block_ui

function sn_fix_dialog_close(event, ui) {
  jQuery('.ui-widget-overlay').bind('click', function(){ jQuery('#' + event.target.id).dialog('close'); });
} // sn_fix_dialog_close

function sn_unblock_ui(content_el) {
  jQuery('html.wp-toolbar').removeClass('sn-overlay-active');
  jQuery('#wpadminbar').removeClass('sn-overlay-active');
  jQuery('#sn_overlay').hide();

  if (content_el) {
    jQuery(content_el, '#sn_overlay').hide();
  }
} // sn_block_ui


jQuery(document).ready(function($) {
  $('#test-details-dialog').dialog({'dialogClass': 'wp-dialog sn-dialog',
                               'modal': true,
                               'resizable': false,
                               'zIndex': 9999,
                               'width': 750,
                               'height': 'auto',
                               'hide': 'fade',
                               'open': function(event, ui) { sn_fix_dialog_close(event, ui); },
                               'close': function(event, ui) { jQuery('#test-details-dialog').html('<p>Please wait.</p>') },
                               'show': 'fade',
                               'autoOpen': false,
                               'closeOnEscape': true
                              });

  // init tabs
  $('#tabs').tabs({
    activate: function( event, ui ) {
        $.cookie('sn_tabs_selected', $('#tabs').tabs('option', 'active'));
    },
    active: $('#tabs').tabs({ active: $.cookie('sn_tabs_selected') })
  });

  // run tests, via ajax
  $('#run-tests').click(function(){
    var data = {action: 'sn_run_tests', '_ajax_nonce': wf_sn.nonce_run_tests};

    sn_block_ui('#sn-site-scan');

    $.post(ajaxurl, data, function(response) {
      if (response != '1') {
        alert('Undocumented error. Page will automatically reload.');
        window.location.reload();
      } else {
        window.location.reload();
      }
    });
  }); // run tests

  // show test details/help/fix dialog
  $('#sn_tests .sn-details a.button').live('click', function(e){
    e.preventDefault();

    test_id = $(this).data('test-id');
    name = $('#' + test_id + ' .test_name').text();
    if (name == '') {
      name = 'Unknown test ID';
      content = 'Help is not available for this test. Make sure you have the latest version of Security Ninja installed.';
    } else {
      content = '<span class="ui-helper-hidden-accessible"><input type="text"></span>' + $('#' + test_id + ' .test_description').html();
      content += '<hr><h3>Auto Fixer</h3><div id="auto-fixer-content"></div>';
    }

    $('#test-details-dialog').html(content);
    $('#test-details-dialog').dialog('option', { title: name, test_id: test_id } ).dialog('open');
    $(document).trigger('sn_test_details_dialog_open', [ test_id, $(this).data('test-status') ] );

    return false;
  }); // show test details dialog

  // abort scan by refreshing
  $('#abort-scan').on('click', function(e){
    e.preventDefault();
    if (confirm('Are you sure you want to stop?')) {
      window.location.reload();
    }
  }); // abort scan


  $('#submit-license').on('click',function(e){
    e.preventDefault();
    $(this).addClass('loading');

     $.post(ajaxurl, {
       action:'sn_activate',
       license_key: $('#license_key').val(),
       _ajax_nonce: wf_sn.nonce_activate_license_key,
       _rand: Math.floor(Math.random() * 9999) + 1
     },
     function(response) {
       if(response.success){
        location.reload();
       } else if(response.data.error_code == '404') {
        verify_licence_ajax($('#license_key').val());
       } else {
        if(response.data.error){
          alert(response.data.error);
        } else if(response.data){
          alert(response.data);
        } else {
          alert('Undocumented error. Please reload the page and try again.');
        }
       }
       $(this).removeClass('loading');
     }).fail(function() {
       alert('Undocumented error. Please reload the page and try again.');
     }).always(function() {
       $(this).removeClass('loading');
     });
  });

  function verify_licence_ajax(licence_code){
    $.get(wf_sn.lc_ep, {
         action:'validate_license',
         license_key: licence_code,
         version: wf_sn.lc_version,
         code_base: 'pro',
         site: wf_sn.lc_site,
         _rand: Math.floor(Math.random() * 9999) + 1
      },
      function(response) {
        if(response.success){
          $.post(ajaxurl, { license_key:licence_code,
                            license_active: response.data.license_active,
                            license_type: response.data.license_type,
                            license_expires: response.data.license_expires,
                            _ajax_nonce: wf_sn.nonce_save_license_key,
                            action: 'sn_activate_ajax'},
          function(response) {
            if (response.success) {              
              location.reload();
            } else {
              alert('Undocumented error. Please reload the page and try again.');
            }
          }).fail(function() {
            alert('Something is not right. Please reload the page and try again');
          }).always(function() {
            $(this).removeClass('loading');
          });

          if(response.data.error){
            alert(response.data.error);
          }
        } else {
          alert(response.data.error);
        }
        $(this).removeClass('loading');
      }).fail(function() {
        alert('Undocumented error. Please reload the page and try again.');
      }).always(function() {
        $(this).removeClass('loading');
      });
  }

  
}); // onload
