<?php
/*
 * Security Ninja
 * (c) 2011 - 2018 Web factory Ltd
 *
 */

class wf_sn_af_fix_rpc_meta extends wf_sn_af {
  static function get_label($label) {
    $labels = array('title' => 'Disable xmlrpc',
					'fixable' => true,
					'info' => 'If you\'re not using any Really Simple Discovery services such as pingbacks there\'s no need to advertise that endpoint (link) in the header. 
					           Please note that for most sites this is not a security issue because they "want to be discovered" but if you want to hide the fact that you\'re using WP this fix will disable it.',
					'msg_ok' => 'Disabled Sucessfuly',
					'msg_bad' => 'Disable Failed');  
    if(!array_key_exists($label,$labels)){
	  return '';	
	} else {	
	  return $labels[$label];
	}
  }
  					
  static function fix() {
    $backup_timestamp=time();
	$msg = '';
	$error = false;  
      	
    $htaccess_contents = array();
	
	$htaccess_paths = array(ABSPATH . '.htaccess');   
	//Determine if wordpress is in a subdirectory. If that is the case we have to edit the copy of .htaccess in the directory one level up as well
	$siteurl = home_url();
    $wpurl = site_url();
	
	//Determine what subdirectory wordpress is in and check that .htaccess at the ABSPATH location is in that directory as well. if it is we can get the path to the other .htaccess one level up
	$subdirectory = str_replace('/', '', str_replace($siteurl, '', $wpurl));
	if(strpos($htaccess_paths[0], $subdirectory . '/.htaccess') !== false){
	  if(file_exists(dirname(ABSPATH) . '/.htaccess')){
		$htaccess_paths[] = dirname(ABSPATH) . '/.htaccess';
	  } else {
		// if we can't find the htaccess in the directory above something might be wrong
		$msg .= '.htacess file one level up in the directory structure not found.';  
		$error = true;
	  }
	}
	
	if(!$error){
	  foreach($htaccess_paths as $htaccess_fid => $htaccess_path){
		if(!file_exists($htaccess_path)){
		  $msg .= '.htaccess file not found.<br />';
		  $error = true;
		  break;
		}
  
		$htaccess_contents[$htaccess_fid] = @file_get_contents($htaccess_path);
		if(!$htaccess_contents[$htaccess_fid] || strlen($htaccess_contents[$htaccess_fid]) === false){
		  $msg .= 'Cannot read .htaccess file.<br />';
		  $error = true;
		  break;
		}
		
		if(strpos($htaccess_contents[$htaccess_fid], '<Files xmlrpc.php>') !== false){
		  $msg .= 'A rule regarding xmlrpc already exists in your .htaccess. Please reanalyze your website to update the test status.<br />';	
		  $error = true;
		  break;
		}
		
		$new_htaccess = $htaccess_contents[$htaccess_fid] . PHP_EOL . '<Files xmlrpc.php>' . PHP_EOL . 'Order Deny,Allow' . PHP_EOL . 'Deny from all' . PHP_EOL . '</Files>';
		
		if( file_put_contents($htaccess_path, $new_htaccess, LOCK_EX) === false){
		  $msg .= 'Could not write to .htaccess.<br />';	
		  $error = true;
		  break;
		}
	  }
	}
	
	// if editing .htacess went OK, update wp-config and option key
	if(!$error){
		wf_sn_af::backup_file(wf_sn_af::$wp_config_path,$backup_timestamp,'rpc_meta');	
			
		$config_file = file(wf_sn_af::$wp_config_path);	  
		foreach($config_file as $line_num => $line){
		  if(strpos(str_replace(' ', '', str_replace('"', '\'', $line)), 'require_once(ABSPATH.\'wp-settings.php\');') !== false){
			$wp_settings_require_line = $line_num+1;
			break;
		  }
		}
		array_splice($config_file, $wp_settings_require_line, 0, PHP_EOL . 'add_filter(\'xmlrpc_enabled\', \'__return_false\');');	  
		file_put_contents(wf_sn_af::$wp_config_path, implode('', $config_file), LOCK_EX);  		
		
		wf_sn_af::update_option('sn-hide-rpc-meta', true);
	}
    
    $no_wsod = wf_sn_af::test_wordpress_status();	
    if(!$no_wsod || $error){
	  // if anyting went wrong restore everything
      wf_sn_af::backup_file_restore(wf_sn_af::$wp_config_path, $backup_timestamp, 'rpc_meta');
	  wf_sn_af::update_option('sn-hide-rpc-meta', false); 
	  foreach($htaccess_paths as $htaccess_fid => $htaccess_path){
        // check if we have read the htaccess files, so we don't write empty
		if(array_key_exists($htaccess_fid, $htaccess_contents) && $htaccess_contents[$htaccess_fid] !== false && strlen($htaccess_contents[$htaccess_fid]) !== false){
		  file_put_contents($htaccess_path, $htaccess_contents[$htaccess_fid], LOCK_EX);
		}
	  }
      return $msg . self::get_label('msg_bad');	
    } else {
      wf_sn_af::mark_as_fixed('uploads_browsable');
      return self::get_label('msg_ok');	
    }
  } 
} // wf_sn_af_fix_rpc_meta
