<?php
/*
 * Security Ninja PRO
 * (c) 2018. Web factory Ltd
 */


if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}


define('WF_SN_CF_OPTIONS_KEY', 'wf_sn_cf');
define('WF_SN_CF_IPS_KEY', 'wf_sn_cf_ips');


class wf_sn_cf {
  static $options;


  static function init() {
    self::$options = self::get_options();
    $current_user_ip = self::getUserIP();

    if (!is_admin() && (int) self::$options['active'] === 1) {
      if(isset($_REQUEST['snf']) && $_REQUEST['snf'] == self::$options['unblock_url']){
        wf_sn_el_modules::log_event('security_ninja', 'unblocked_ip', 'New IP added to the whitelist using the secret access URL.', '');
        if (!in_array($current_user_ip, self::$options['whitelist'])) {
          self::$options['whitelist'][] = $current_user_ip;
          update_option(WF_SN_CF_OPTIONS_KEY, self::$options);
        }
      }

      add_action('login_init', array(__CLASS__, 'form_init_check'));
      add_filter('authenticate', array(__CLASS__, 'login_filter'), 10, 3);

      if ((int) self::$options['global'] === 1) {
        if (self::is_banned_ip()){
          wf_sn_el_modules::log_event('security_ninja', 'blocked_ip', 'Visitor blocked from accessing the site.', '');
          wp_die(self::$options['message']);
        }
      }
    } // not admin and active

    add_filter('login_message', array(__CLASS__, 'login_message'));
    add_action('wp_login_failed', array(__CLASS__, 'failed_login'));

    if (is_admin()) {
      // add tab to Security Ninja tabs
      add_filter('sn_tabs', array(__CLASS__, 'sn_tabs'));

      // enqueue scripts
      add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

      // settings registration
      add_action('admin_init', array(__CLASS__, 'register_settings'));

      // add custom text for GUI overlay
      add_action('sn_overlay_content', array(__CLASS__, 'overlay_content'));

      // register ajax endpoints
      add_action('wp_ajax_sn_enable_firewall', array(__CLASS__, 'ajax_enable_firewall'));
      add_action('wp_ajax_sn_clear_blacklist', array(__CLASS__, 'ajax_clear_blacklist'));
      add_action('wp_ajax_sn_send_unblock_email', array(__CLASS__, 'ajax_send_unblock_email'));
      add_action('wp_ajax_sn_test_ip', array(__CLASS__, 'ajax_test_ip'));

    } // if admin
  } // init

  static function get_options() {
    $defaults = array('active' => '0',
                      'global' => 'all',
                      'whitelist' => array(self::getUserIP()),
                      'banned_ips' => array(),
                      'max_login_attempts' => 5,
                      'max_login_attempts_time' => 5,
                      'bruteforce_ban_time' => 120,
                      'login_msg' => 'Warning: multiple failed login attempts will get you banned.',
                      'message' => 'Sorry but the website is not available from your current location.');
    $options = get_option(WF_SN_CF_OPTIONS_KEY, array());

    $options = array_merge($defaults, $options);

    return $options;
  } // get_options


  static function ajax_enable_firewall() {
    self::$options['active'] = 1;
    update_option(WF_SN_CF_OPTIONS_KEY, self::$options);

    echo '1';
    die();
  } // ajax_enable_firewall
  

  static function ajax_test_ip() {
    $ip = trim(@$_GET['ip']);

    if (!filter_var($ip, FILTER_VALIDATE_IP)) {
      wp_send_json_success( 'Please enter a valid IP address to test.' );
    }

    if (self::is_banned_ip($ip)) {
      wp_send_json_success( 'IP ' . $ip . ' is banned.' );
    } else {
      wp_send_json_success( 'IP ' . $ip . ' is not banned.' );
    }
  } // ajax_enable_firewall


  static function ajax_clear_blacklist() {
    self::$options['banned_ips'] = array();
    update_option(WF_SN_CF_OPTIONS_KEY, self::$options);

    echo '1';
    die();
  } // ajax_clear_blacklist


  static function ajax_send_unblock_email() {
    if (false === ($email = is_email($_GET['email']))) {
      echo '0';
      die();
    }

    if(!(array_key_exists('unblock_url', self::$options) && strlen(self::$options['unblock_url']) == 32)){
      self::$options['unblock_url'] = md5(time());
      update_option(WF_SN_CF_OPTIONS_KEY,self::$options);
    }

    $subject = 'Security Ninja Firewall secret access link';
    $body = 'Your secret access link is: ' . home_url('/?snf=' . self::$options['unblock_url']);
    $body .= "\nPlease keep it safe and do not share it with others. Use it only if you get blocked by the firewall.";

    if(wp_mail($_GET['email'], $subject, $body)) {
      echo '1';
    } else {
      echo '0';
    }

    die();
  } // ajax_send_unblock_email


  static function form_init_check() {
    if(self::is_banned_ip()){
      wf_sn_el_modules::log_event('security_ninja', 'blocked_ip', 'Visitor blocked from accessing the login page.', '');
      wp_clear_auth_cookie();
      wp_redirect(home_url());

      die();
      return false;
    }
  } // form_init_check


  static function login_filter($user, $username, $password) {
    if (self::is_banned_ip()){
      wp_die(self::$options['message']);
    }
    return $user;
  } // login_filter


  static function prune_banned() {
    $update = false;

    if(!array_key_exists('banned_ips',self::$options)){
      self::$options['banned_ips'] = array();
    }

    foreach (self::$options['banned_ips'] as $ip => $time) {
      if ($time < current_time('timestamp')) {
        unset(self::$options['banned_ips'][$ip]);
        $update = true;
      }
    }

    if ($update) {
      update_option(WF_SN_CF_OPTIONS_KEY, self::$options);
    }
  } // prune_banned


  // log failed login
  static function failed_login($username) {
    global $wpdb;

    if(!array_key_exists('banned_ips',self::$options)){
      self::$options['banned_ips'] = array();
    }

    $current_user_ip = self::getUserIP();

    $date = date('Y-m-d H:i:m', current_time('timestamp'));
    $query = $wpdb->prepare('SELECT COUNT(id) FROM ' . $wpdb->prefix . WF_SN_EL_TABLE .
                            ' WHERE ip = %s AND action = %s AND timestamp >= DATE_SUB(%s, INTERVAL %s MINUTE)',
                            $current_user_ip, 'wp_login_failed', $date, self::$options['max_login_attempts_time']);
    $login_attempts = $wpdb->get_var($query);

    if ($login_attempts >= self::$options['max_login_attempts'] && !isset(self::$options['banned_ips'][$current_user_ip])) {
      self::$options['banned_ips'][$current_user_ip] = current_time('timestamp') + self::$options['bruteforce_ban_time'] * 60;
      wf_sn_el_modules::log_event('security_ninja', 'firewall_ip_banned', 'IP banned due to multiple failed login attempts.', '');
    }

    update_option(WF_SN_CF_OPTIONS_KEY, self::$options);

    $ban = self::is_banned_ip();

    if ($ban && is_user_logged_in()) {
      wf_sn_el_modules::log_event('security_ninja', 'login_denied_banned_IP', 'Visitor blocked from logging in.', '');
      wp_clear_auth_cookie();
      wp_redirect(home_url());
      die();
    }
  } // failed_login

  static function ipCIDRCheck($IP, $CIDR) {
    list ($net, $mask) = explode ('/', $CIDR);
    $ip_net = ip2long ($net);
    @$ip_mask = ~((1 << (32 - $mask)) - 1);
    $ip_ip = ip2long ($IP);
    $ip_ip_net = $ip_ip & $ip_mask;
    return ($ip_ip_net == $ip_net);
  } // ipCIDRCheck

  static function is_banned_ip($ip = false) {
    self::prune_banned();

    if ($ip) {
      $current_user_ip = $ip;
    } else {
      $current_user_ip = self::getUserIP();
    }
    $ips = self::get_ips_list();

    if(!array_key_exists('banned_ips', self::$options)) {
      self::$options['banned_ips'] = array();
    }

    if (is_array(self::$options['whitelist']) && self::is_whitelisted($current_user_ip, self::$options['whitelist'])) {
      return false;
    } elseif (array_key_exists($current_user_ip, self::$options['banned_ips'])){
      return true;
    } elseif (in_array($current_user_ip, $ips['ips'])){
      return true;
    } else {
      $nework_array = explode('.',$current_user_ip, 2);
      if(array_key_exists($nework_array[0],$ips['subnets'])){
        foreach($ips['subnets'][$nework_array[0]] as $subnet){
          if(self::ipCIDRCheck($current_user_ip,$subnet)){
            return true;
          }
        }
      }
    }
    return false;
  } // is_banned_ip

  static function is_whitelisted($ip, $whitelist){
    foreach($whitelist as $wip){
      if(strpos($wip,'/') !== false){
        if(self::ipCIDRCheck($ip,$wip)){
          return true;
        }
      } else {
        if($ip == $wip){
          return true;
        }
      }
    }
    return false;
  }

  static function get_ips_list(){
    $sn_firewall_data = get_option(WF_SN_CF_IPS_KEY);

    if(!(is_array($sn_firewall_data) && array_key_exists('timestamp', $sn_firewall_data))){
      $sn_firewall_data = array();
      $sn_firewall_data['timestamp'] = 0;
    }

    if ($sn_firewall_data['timestamp'] < current_time('timestamp') - 6*HOUR_IN_SECONDS) {
      $sn_options = wf_sn::get_options();
      $request_params = array('sslverify' => false, 'timeout' => 25, 'redirection' => 2);
      $request_args = array('action' => 'refresh_firewall',
                            'license_key' => $sn_options['license_key'],
                            'code_base' => 'pro',
                            'version' => wf_sn::$version,
                            'site' => get_home_url());

      $url = add_query_arg($request_args, wf_sn::$licensing_servers[0]);
      $response = wp_remote_get(esc_url_raw($url), $request_params);

      if (is_wp_error($response) || !wp_remote_retrieve_body($response)) {
        $url = add_query_arg($request_args, wf_sn::$licensing_servers[1]);
        $response = wp_remote_get(esc_url_raw($url), $request_params);
      }

      if (!is_wp_error($response) && wp_remote_retrieve_body($response)) {
        $result = wp_remote_retrieve_body($response);
        $ips = json_decode($result, true);
        if(array_key_exists('ips',$ips)){
          $sn_firewall_data = $ips;
          wf_sn_el_modules::log_event('security_ninja', 'firewall_rules', 'The cloud firewall IP block list has been updated.', '');

        }
      }

      //Update the timestamp even if request failed so we don't refresh the list on every load
      $sn_firewall_data['timestamp'] = current_time('timestamp');
      update_option( WF_SN_CF_IPS_KEY, $sn_firewall_data);
    }
    return $sn_firewall_data;
  }

  static function enqueue_scripts() {
    if (wf_sn::is_plugin_page()) {
      $plugin_url = plugin_dir_url(__FILE__);

      wp_enqueue_style('sn-cf-css', $plugin_url . 'css/wf-sn-cf.css', array(), wf_sn::$version);
      wp_enqueue_script('sn-cf-js', $plugin_url . 'js/wf-sn-cf.js', array(), wf_sn::$version, true);
    } // if
  } // enqueue_scripts


  static function register_settings() {
    register_setting(WF_SN_CF_OPTIONS_KEY, 'wf_sn_cf', array(__CLASS__, 'sanitize_settings'));
  } // register_settings


  static function getUserIP(){
      $client  = @$_SERVER['HTTP_CLIENT_IP'];
      $forward = @$_SERVER['HTTP_X_FORWARDED_FOR'];
      $remote  = $_SERVER['REMOTE_ADDR'];

      if (filter_var($client, FILTER_VALIDATE_IP)) {
        $ip = $client;
      } elseif (filter_var($forward, FILTER_VALIDATE_IP)) {
        $ip = $forward;
      } else {
        $ip = $remote;
      }

      return $ip;
  } // getUserIP


  // display message above login form
  static function login_message($msg) {
    if (self::is_active() && self::$options['login_msg'] &&
       @$_GET['action'] != 'register' &&
       @$_GET['action'] != 'lostpassword') {
      $msg = '<p class="message register">' . self::$options['login_msg'] . '</p>' . $msg;
    }

    return $msg;
  } // login message


  // sanitize settings on save
  static function sanitize_settings($values) {
    $old_options = get_option(WF_SN_CF_OPTIONS_KEY);

    $old_options['active'] = 0;
    $old_options['global'] = 0;

    foreach ($values as $key => $value) {
      switch ($key) {
        case 'active':
        case 'global':
        case 'message':
        case 'max_login_attempts':
        case 'max_login_attempts_time':
        case 'bruteforce_ban_time':
        case 'login_msg':
        case 'unblock_url':
          $values[$key] = trim($value);
        break;
        case 'whitelist':
          if(!is_array($values[$key])){
            $values[$key] = explode(PHP_EOL, $values[$key]);
          }
        break;
      } // switch
    } // foreach

    // temporarily disabled
    $user_ip = self::getUserIP();
    if (false && $values['active'] == 1) {
      if (!in_array($user_ip, $values['whitelist'])) {
        $values['whitelist'][] = $user_ip;
      }
    }

    return array_merge($old_options, $values);
  } // sanitize_settings


  // add new tab
  static function sn_tabs($tabs) {
    $core_tab = array('id' => 'sn_cf', 'class' => '', 'label' => 'Cloud Firewall', 'callback' => array(__CLASS__, 'do_page'));
    $done = 0;

    for ($i = 0; $i < sizeof($tabs); $i++) {
      if ($tabs[$i]['id'] == 'sn_cf') {
        $tabs[$i] = $core_tab;
        $done = 1;
        break;
      }
    } // for

    if (!$done) {
      $tabs[] = $core_tab;
    }

    return $tabs;
  } // sn_tabs


  static function get_table_prefix() {
    global $wpdb;

    if (is_multisite() && !defined('MULTISITE')) {
      $table_prefix = $wpdb->base_prefix;
    } else {
      $table_prefix = $wpdb->get_blog_prefix(0);
    }

    return $table_prefix;
  } // get_table_prefix


  static function create_toogle_switch($name, $options = array(), $output = true) {
    $default_options = array('value' => '1', 'saved_value' => '', 'option_key' => $name);
    $options = array_merge($default_options, $options);

    $out = "\n";
    $out .= '<div class="toggle-wrapper">';
    $out .= '<input type="checkbox" id="' . $name . '" ' . self::checked($options['value'], $options['saved_value']) . ' type="checkbox" value="' . $options['value'] . '" name="' . $options['option_key'] . '">';
    $out .= '<label for="' . $name . '" class="toggle"><span class="toggle_handler"></span></label>';
    $out .= '</div>';

    if ($output) {
      echo $out;
    } else {
      return $out;
    }
  } // create_toggle_switch


  static function checked($value, $current, $echo = false) {
    $out = '';

    if (!is_array($current)) {
      $current = (array) $current;
    }

    if (in_array($value, $current)) {
      $out = ' checked="checked" ';
    }

    if ($echo) {
      echo $out;
    } else {
      return $out;
    }
  } // checked


  static function create_select_options($options, $selected = null, $output = true) {
    $out = "\n";

    foreach ($options as $tmp) {
      if ($selected == $tmp['val']) {
        $out .= "<option selected=\"selected\" value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      } else {
        $out .= "<option value=\"{$tmp['val']}\">{$tmp['label']}&nbsp;</option>\n";
      }
    }

    if($output) {
      echo $out;
    } else {
      return $out;
    }
  } //  create_select_options

  // add custom message to overlay
  static function overlay_content() {
    echo '<div id="sn-cloud-firewall" style="display: none; text-align:center;">';
    echo '<h2 style="font-weight: bold;">Important! Please READ! Not the usual mumbo-jumbo.</h2>';
    echo '<p>In an unlikely situation that your IP gets banned, you will not be able to login or access the site. In that case you need the secret access link. It whitelists your IP and enables access. Please store the link in a safe place or use the form below to get it sent to your email address.<br><br><code>' . home_url('/?snf='.self::$options['unblock_url']) . '</code></p>';

    echo '<p><br><br>Enter your email below to receive the secret access link in case you get locked out:<br />';
    echo '<input style="width: 250px;" type="text" id="sn-ublock-email" name="sn-ublock-email" value="" placeholder="john@example.com"><br />
    <p id="sn-unblock-message"></p>';
    echo '<input type="button" value="Send secret access link" id="sn-send-unlock-code" class="input-button" /></p>';

    echo '<p><br><input type="button" value="Close (3)" id="sn-close-firewall" class="input-button button-primary" /></p>';

    echo '</div>';
  } // overlay_content

  static function is_active() {
    return (bool) self::$options['active'];
  } // is_active

  // display results
  static function do_page() {
    global $wpdb;

    $ips = self::get_ips_list();

    $ips_count = count($ips['ips']);
    $subnets_count = 0;
    foreach($ips['subnets'] as $subnet){
      $subnets_count+=count($subnet);
    }

    if (!(array_key_exists('unblock_url', self::$options) && strlen(self::$options['unblock_url']) == 32)) {
      self::$options['unblock_url'] = md5(time());
      update_option(WF_SN_CF_OPTIONS_KEY,self::$options);
    }


    if(!array_key_exists('total', $ips)){
      $total_ips = 0;
      foreach($ips['subnets'] as $prefix => $subnet){
        foreach($subnet as $sub){
          $mask = explode('/',str_replace('\'','',$sub));
          $total_ips += pow(2, 32-$mask[1])-2;
        }
      }
      $ips['total'] = $total_ips + count($ips['ips']) + count(self::$options['banned_ips']);
      update_option( WF_SN_CF_IPS_KEY, $ips);
    }

    echo '<div class="submit-test-container">';
      echo '<div id="counters">';
      if (self::is_active()) {
        echo '<span class="score" style="border-left: none;">' . number_format($ips['total']) . '<br><i>IPs banned by cloud firewall</i></span>';
        echo '<span class="score">' . number_format(count(self::$options['banned_ips'])) . '<br><i>IPs banned by local firewall</i></span>';
      } else {
        echo '<span class="score" style="border-left: none;">' . number_format($ips['total']) . '<br><i>IPs in cloud firewall bad IPs database</i></span>';
        echo '<span class="score">' . number_format(count(self::$options['banned_ips'])) . '<br><i>IPs in local bad IPs database</i></span>';
      }
      echo '</div>';

      echo '<p>Cloud Firewall is a dynamic, continuously changing database of bad IP addresses updated every six hours. It contains roughly 600 million IPs that are known for distributing malware, performing brute force attacks on sites and doing other "bad" activities. The database is created by analysing log files of millions of sites.<br>By using the cloud firewall, you will be one step ahead of the bad guys. They won\'t be able to login to your site or access it at all (if you enable that option). You are not sharing any data with the Cloud Firewall, just using its list.<br>The local firewall protects your login from brute force attacks. Anybody who fails to log in several times in a given period will be banned.</p>';
      echo '<br /><br />';

      if((int) self::$options['active'] === 1) {
        echo '<input type="button" value="Disable Firewall" id="sn-disable-firewall" style="background: #cc0000;" class="input-button red button-primary" />';
      } else {
        echo '<input type="button" value="Enable Firewall" id="sn-enable-firewall-overlay" class="input-button button-primary" />';
      }
    echo '</div>';

    if ((int)self::$options['active'] === 1) {
      echo '<form action="options.php" id="sn-firewall-settings-form" method="post">';
      settings_fields('wf_sn_cf');

      echo '<h3 class="ss_header">Cloud Firewall Settings</h3>';
      echo '<table class="form-table" id="sn-cf-options"><tbody>';

      echo '<tr valign="top">
      <th scope="row"><label for="wf_sn_cf_global">Prevent Banned IPs from Accessing the Site</label></th>
      <td class="sn-cf-options">';
        self::create_toogle_switch(WF_SN_CF_OPTIONS_KEY . '_global', array('saved_value' => self::$options['global'], 'option_key' => WF_SN_CF_OPTIONS_KEY. '[global]'));
        echo '<span class="description">If set to ON cloud and local firewall will prevent banned IPs from accessing the site all together. If set to OFF they will not be able to log in, but will be able to view the site. Unless you are experiencing heavy traffic from bad IPs that\'s messing with your stats we recommend keeping this option OFF. You\'ll still have plenty protection.</span>';
      echo '</td></tr>';

      echo '<tr valign="top">
      <th scope="row"><label for="wf_sn_cf_message">Message for banned IPs</label></th>
      <td class="sn-cf-options"><textarea id="wf_sn_cf_message" name="' . WF_SN_CF_OPTIONS_KEY . '[message]" rows="3" cols="50">' .self::$options['message'].'</textarea><span class="description">Message shown to banned IPs when they try to access the site or log in.</span></td></tr>';

      for ($i = 2; $i <= 10; $i++) {
        $max_login_attempts[] = array('val' => $i, 'label' => $i);
      }
      for ($i = 2; $i <= 15; $i++) {
        $max_login_attempts_time_s[] = array('val' => $i, 'label' => $i);
      }

      $bruteforce_timeouts[] = array('val' => 2, 'label' => '2 minutes');
      $bruteforce_timeouts[] = array('val' => 10, 'label' => '10 minutes');
      $bruteforce_timeouts[] = array('val' => 20, 'label' => '20 minutes');
      $bruteforce_timeouts[] = array('val' => 30, 'label' => '30 minutes');
      $bruteforce_timeouts[] = array('val' => 60, 'label' => '1 hour');
      $bruteforce_timeouts[] = array('val' => 120, 'label' => '2 hours');
      $bruteforce_timeouts[] = array('val' => 1440, 'label' => '1 day');
      $bruteforce_timeouts[] = array('val' => 2880, 'label' => '2 days');
      $bruteforce_timeouts[] = array('val' => 10080, 'label' => '7 days');
      $bruteforce_timeouts[] = array('val' => 43200, 'label' => '1 month');
      $bruteforce_timeouts[] = array('val' => 525600, 'label' => '1 year');
      $bruteforce_timeouts[] = array('val' => 5256000, 'label' => 'forever');

      $notification_settings[] = array('val' => 0, 'label' => 'No');
      $notification_settings[] = array('val' => 1, 'label' => 'Yes');


      echo '<tr valign="top">';
      echo '<th scope="row"><label for="wf_sn_options_mla">Auto-ban rules for failed login attempts</label></th>';
      echo '<td class="'.(!self::$options['active']?'sn-disabled':'').' sn-cf-options">
            <select name="'.WF_SN_CF_OPTIONS_KEY.'[max_login_attempts]" id="wf_sn_options_mla">';
            self::create_select_options($max_login_attempts, self::$options['max_login_attempts']);
      echo '</select> failed login attempts in ';
      echo '<select name="'.WF_SN_CF_OPTIONS_KEY.'[max_login_attempts_time]" id="wf_sn_options_mlat">';
            self::create_select_options($max_login_attempts_time_s, self::$options['max_login_attempts_time']);
      echo '</select> minutes get the IP banned for ';
      echo '<select name="' . WF_SN_CF_OPTIONS_KEY . '[bruteforce_ban_time]" id="wf_sn_options_bbt">';
            self::create_select_options($bruteforce_timeouts, self::$options['bruteforce_ban_time']);
      echo '</select>';
      echo '<span class="description">Users who continuously make failed login attempts will get banned. Five failed attempts in five minutes is a good threshold.</span>';
      echo '</td></tr>';

      echo '<tr valign="top">';
      echo '<th scope="row"><label for="wf_sn_options_login_msg">Login notice</label></th>';
      echo '<td class="sn-cf-options">
            <textarea cols="50" rows="3" name="' . WF_SN_CF_OPTIONS_KEY . '[login_msg]" id="wf_sn_options_login_msg">' . self::$options['login_msg'] . '</textarea>';
      echo '<span class="description">Usefull on a multi-user site to warn people what happens if they fail to login too many times.</span>';
      echo '</td>';
      echo '</tr>';

      echo '<tr valign="top">
      <th scope="row"><label for="wf_sn_cf_whitelist">Whitelisted IPs</label></th>
      <td class="sn-cf-options"><textarea id="wf_sn_cf_whitelist" name="' . WF_SN_CF_OPTIONS_KEY . '[whitelist]" rows="5" cols="50">'.(is_array(self::$options['whitelist'])?implode(PHP_EOL,self::$options['whitelist']):'') . '</textarea>';
      echo '<span class="description">Write one IP or subnet mask per line. Your IP address is: ' . self::getUserIP() . '</span>';
      echo '</td></tr>';

      echo '<tr valign="top">
        <th scope="row"><label for="">Locally Banned IPs</label></th>
        <td id="sn-firewall-blacklist">';
        self::prune_banned();
        if(count(self::$options['banned_ips'])){
          echo '<ul style="margin-top: 5px;">';
          foreach(self::$options['banned_ips'] as $banned_ip => $ban_time){
              echo '<li>' . $banned_ip. '; time till ban expires: ' . human_time_diff(current_time('timestamp'), $ban_time) . '</li>';
          }
          echo '</ul>';
          echo '<br><input type="button" value="Clear list of banned IPs" id="sn-firewall-blacklist-clear" style="background: #cc0000;" class="input-button red button-primary" />';
        } else {
          echo 'No locally banned IPs';
        }
      echo '<span class="description">IPs banned due to too many failed login attempts.</span>';
      echo '</td></tr>';

      echo '<tr valign="top">
        <th scope="row"><label for="">Secret Access URL</label></th>
        <td id="sn-firewall-blacklist"><code>';
      echo home_url('/?snf=' . self::$options['unblock_url']);
      echo '</code>';
      echo '<span class="description">Do not share this URL! Use it only to access the site if your IP gets banned.</span>';
      echo '</td></tr>';

      echo '<tr valign="top">
        <th scope="row"><label for="wf-cf-ip-test">Test IP</label></th>
        <td id="sn-firewall-test-ip">';
      echo '<input type="text" placeholder="IP address, ie: 213.45.66.12" class="regular-text" value="" id="wf-cf-ip-test">&nbsp; &nbsp;';
      echo '<a href="#" id="wf-cf-do-test-ip" class="button js-action">Test if IP is Banned</a>';
      echo '<span class="description">Enter an IP address in order to test if it\'s banned. Your IP address is: ' . self::getUserIP() . '</span>';
      echo '</td></tr>';

      echo '<input type="hidden" id="wf_sn_cf_active" name="wf_sn_cf[active]" value="' . self::$options['active'] . '" />';
      echo '<tr valign="top"><td colspan="2" style="padding:0px;">';
      echo '<p class="submit"><br><input type="submit" value="Save Changes" class="input-button button-primary" name="Submit" /></p>';
      echo '</td></tr>';

      echo '</table>';
      echo '</form>';
    }
  } // do_page

  // clean-up when deactivated
  static function deactivate() {
    delete_option(WF_SN_CF_OPTIONS_KEY);
    delete_option(WF_SN_CF_IPS_KEY);
  } // deactivate
} // wf_sn_cf class


// hook everything up
add_action('plugins_loaded', array('wf_sn_cf', 'init'));

// when deativated clean up
register_deactivation_hook(WF_SN_BASE_FILE, array('wf_sn_cf', 'deactivate'));