<?php
/*
Plugin Name: Security Ninja PRO
Plugin URI: https://wpsecurityninja.com/
Description: Check your site for <strong>security vulnerabilities</strong> and get precise suggestions for corrective actions on passwords, user accounts, file permissions, database security, version hiding, plugins, themes and other security aspects.
Author: Web factory Ltd
Version: 5.42
Author URI: http://www.webfactoryltd.com/
*/


// this is an include only WP file
if (!defined('ABSPATH')) {
  die;
}

// constants
define('WF_SN_PLUGIN_DIR', plugin_dir_path(__FILE__));
define('WF_SN_PLUGIN_URL', plugin_dir_url(__FILE__));
define('WF_SN_BASE_FILE', __FILE__);
define('WF_SN_RESULTS_KEY', 'wf_sn_results');
define('WF_SN_OPTIONS_KEY', 'wf_sn_options');
define('WF_SN_POINTERS_KEY', 'wf_sn_pointers');
define('WF_SN_MAX_EXEC_SEC', 200);


require_once WF_SN_PLUGIN_DIR . 'sn-tests.php';

require_once WF_SN_PLUGIN_DIR . 'modules/core-scanner/core-scanner.php';
require_once WF_SN_PLUGIN_DIR . 'modules/malware-scanner/malware-scanner.php';
require_once WF_SN_PLUGIN_DIR . 'modules/events-logger/events-logger.php';
require_once WF_SN_PLUGIN_DIR . 'modules/scheduled-scanner/scheduled-scanner.php';
require_once WF_SN_PLUGIN_DIR . 'modules/auto-fixer/auto-fixer.php';
require_once WF_SN_PLUGIN_DIR . 'modules/database-optimizer/database-optimizer.php';
require_once WF_SN_PLUGIN_DIR . 'modules/cloud-firewall/cloud-firewall.php';


class wf_sn {
  static $version = 0;
  static $licensing_servers = array('https://license1.wpsecurityninja.com/', 'http://license2.wpsecurityninja.com/');
  static $skip_tests = array();


  // get plugin version from header
  static function get_plugin_version() {
    $plugin_data = get_file_data(__FILE__, array('version' => 'Version'), 'plugin');
    wf_sn::$version = $plugin_data['version'];

    return $plugin_data['version'];
  } // get_plugin_version


  // init plugin
  static function init() {
    // SN requires WP v4.4
    if (!version_compare(get_bloginfo('version'), '4.4',  '>=')) {
      add_action('admin_notices', array(__CLASS__, 'min_version_error'));
      return;
    }

    // does the user have enough privilages to use the plugin?
    if (is_admin() && current_user_can('administrator')) {
      // add menu item to tools
      add_action('admin_menu', array(__CLASS__, 'admin_menu'));

      // aditional links in plugin description
      add_filter('plugin_action_links_' . basename(dirname(__FILE__)) . '/' . basename(__FILE__),
                 array(__CLASS__, 'plugin_action_links'));
      add_filter('plugin_row_meta', array(__CLASS__, 'plugin_meta_links'), 10, 2);

      // enqueue scripts
      add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

      // register settings key
      add_action('admin_init', array(__CLASS__, 'register_settings'));

      // register endpoints
      add_action('wp_ajax_sn_run_tests', array(__CLASS__, 'run_tests'));
      add_action('wp_ajax_sn_dismiss_pointer', array(__CLASS__, 'dismiss_pointer_ajax'));
      add_action('admin_action_sn_verify_license', array(__CLASS__, 'verify_license_action'));
      add_action('wp_ajax_sn_activate', array(__CLASS__, 'activate_license_key_ajax'));
      add_action('wp_ajax_sn_activate_ajax', array(__CLASS__, 'save_license_key_ajax'));

      // updates
      add_filter('pre_set_site_transient_update_plugins', array(__CLASS__, 'update_filter'));
      add_filter('plugins_api', array(__CLASS__, 'update_details'), 10, 3);

      // warn if tests were not run
      add_action('admin_notices', array(__CLASS__, 'run_tests_warning'));

      // warn if Wordfence is active
      add_action('admin_notices', array(__CLASS__, 'wordfence_warning'));

      // license related warnings
      add_action('admin_notices', array(__CLASS__, 'license_warning'));

      // add_markup for UI overlay
      add_action('admin_footer', array(__CLASS__, 'admin_footer'));

    } // if is_admin
  } // init


  // some things have to be loaded earlier
  static function plugins_loaded() {
    wf_sn::get_plugin_version();
  } // plugins_loaded


  // add links to plugin's description in plugins table
  static function plugin_meta_links($links, $file) {
    $support_link = '<a target="_blank" href="mailto:securityninja@webfactoryltd.com?subject=Security%20Ninja%20PRO%20problem" title="Problems? We\'re here to help">Support</a>';

    if ($file == plugin_basename(__FILE__)) {
      $links[] = $support_link;
    }

    return $links;
  } // plugin_meta_links


  // add settings link to plugins page
  static function plugin_action_links($links) {
    $settings_link = '<a href="tools.php?page=wf-sn" title="Security Ninja">Secure the Site</a>';
    array_unshift($links, $settings_link);

    return $links;
  } // plugin_action_links


  // test if we're on plugin's page
  static function is_plugin_page() {
    $current_screen = get_current_screen();

    if ($current_screen->id == 'tools_page_wf-sn') {
      return true;
    } else {
      return false;
    }
  } // is_plugin_page


  // enqueue CSS and JS scripts on plugin's pages
  static function enqueue_scripts() {
    if (wf_sn::is_plugin_page()) {
      wp_enqueue_script('jquery-ui-tabs');
      wp_enqueue_script('sn-jquery-plugins', WF_SN_PLUGIN_URL . 'js/sn-jquery-plugins.js', array('jquery'), wf_sn::$version, true);
      wp_enqueue_script('sn-js', WF_SN_PLUGIN_URL . 'js/sn-common.js', array('jquery'), wf_sn::$version, true);
      wp_enqueue_style('sn-css', WF_SN_PLUGIN_URL . 'css/sn-style.css', array(), wf_sn::$version);

      wp_dequeue_style('uiStyleSheet');
      wp_dequeue_style('wpcufpnAdmin' );
      wp_dequeue_style('unifStyleSheet' );
      wp_dequeue_style('wpcufpn_codemirror');
      wp_dequeue_style('wpcufpn_codemirrorTheme');
      wp_dequeue_style('collapse-admin-css');
      wp_dequeue_style('jquery-ui-css');
      wp_dequeue_style('tribe-common-admin');
      wp_dequeue_style('file-manager__jquery-ui-css');
      wp_dequeue_style('file-manager__jquery-ui-css-theme');
      wp_dequeue_style('wpmegmaps-jqueryui');
      wp_dequeue_style('facebook-plugin-css');
      wp_dequeue_style('facebook-tip-plugin-css');
      wp_dequeue_style('facebook-member-plugin-css');
      wp_dequeue_style('kc-testimonial-admin');
    } // if SN page

    $pointers = get_option(WF_SN_POINTERS_KEY);
    if (!empty($pointers) && !wf_sn::is_plugin_page()) {
      wp_enqueue_script('wp-pointer');
      wp_enqueue_script('wf-sn-pointers', plugins_url('js/sn-pointers.js', __FILE__), array('jquery'), wf_sn::$version, true);
      wp_enqueue_style('wp-pointer');
      wp_localize_script('wp-pointer', 'wf_sn_pointers', $pointers);
    }

    if (wf_sn::is_plugin_page() || !empty($pointers)) {
      $js_vars = array('sn_plugin_url' => WF_SN_PLUGIN_URL,
                         'nonce_run_tests' => wp_create_nonce('wf_sn_run_tests'),
                         'nonce_refresh_update' => wp_create_nonce('wf_sn_refresh_update'),
                         'nonce_dismiss_pointer' => wp_create_nonce('wf_sn_dismiss_pointer'),
                         'nonce_activate_license_key' => wp_create_nonce('wf_sn_activate_license_key'),
                         'nonce_save_license_key' => wp_create_nonce('wf_sn_save_license_key'),
                         'lc_ep' => wf_sn::$licensing_servers[0],
                         'lc_version' => wf_sn::$version,
                         'lc_site' => get_home_url(),
                         'lc_ip' => $_SERVER['REMOTE_ADDR']);
      wp_localize_script('jquery', 'wf_sn', $js_vars);
    }
  } // enqueue_scripts


  // permanently dismiss pointer
  static function dismiss_pointer_ajax() {
    check_ajax_referer('wf_sn_dismiss_pointer');

    $pointers = get_option(WF_SN_POINTERS_KEY);
    $pointer = trim($_POST['pointer']);

    if (empty($pointers) || empty($pointers[$pointer])) {
      wp_send_json_error();
    }

    unset($pointers[$pointer]);
    update_option(WF_SN_POINTERS_KEY, $pointers);

    wp_send_json_success();
  } // dismiss_pointer_ajax


  // add entry to admin menu
  static function admin_menu() {
    add_management_page('Security Ninja PRO', 'Security Ninja PRO', 'manage_options', 'wf-sn', array(__CLASS__, 'main_page'));
  } // admin_menu


  // display warning if test were never run
  static function run_tests_warning() {
    if (!wf_sn::is_plugin_page() || !wf_sn::is_activated()) {
      return;
    }

    $tests = get_option(WF_SN_RESULTS_KEY, array());

    if (!empty($tests['last_run']) && (current_time('timestamp') - DAY_IN_SECONDS * 30) > $tests['last_run']) {
      echo '<div class="notice notice-error"><p>Security Ninja PRO <strong>tests were not run for more than 30 days.</strong> It\'s advisable to run them once in a while. Click "Analyze Site" to run them now and analyze your site for security vulnerabilities.</p></div>';
    }
  } // run_tests_warning


  // warn if trial is about to expire
  static function license_warning() {
    $_GET['settings-updated']=true;
    
    if (!wf_sn::is_plugin_page()) {
      return;
    }

    $options = self::get_options();
    if ($options['license_type'] == 'trial' && $options['license_expires'] == date('Y-m-d')) {
      echo '<div class="notice notice-error"><p>Your trial license is about to <strong>expire</strong>. If you want to continue using Security Ninja PRO, <a target="_blank" href="' . self::generate_sn_web_link('trial_last_day_notice') . '">purchase a license</a>.</p></div>';
    } elseif ($options['license_type'] == 'trial' && $options['license_expires'] < date('Y-m-d')) {
      echo '<div class="notice notice-error"><p>Your trial license has <strong>expired</strong>. To continue using Security Ninja PRO, <a href="' . self::generate_sn_web_link('trial_expired_notice') . '" target="_blank">purchase a license</a>.</p></div>';
    }
  } // license_warning

  // add_settings_error
  static function add_settings_error($message, $type = 'error', $code = 'wf_sn') {
    global $wp_settings_errors;

    $wp_settings_errors[] = array(
      'setting' => WF_SN_OPTIONS_KEY,
      'code'    => $code,
      'message' => $message,
      'type'    => $type
    );
    set_transient('settings_errors', $wp_settings_errors);
  } // add_settings_error

  // display warning if Wordfence plugin is active
  static function wordfence_warning() {
    if (!wf_sn::is_plugin_page()) {
      return;
    }

    if (defined('WORDFENCE_VERSION') && WORDFENCE_VERSION) {
      echo '<div class="notice notice-error"><p>Please <strong>deactivate Wordfence</strong> before running Security Ninja tests. Some tests are detected as site attacks by Wordfence and hence can\'t be performed properly. Activate Wordfence once you\'re done testing.</p></div>';
    }
  } // wordfence_warning


  // display warning if test were never run
  static function min_version_error() {
    echo '<div class="notice notice-error"><p>Security Ninja PRO <b>requires WordPress version 4.4</b> or higher to function properly. You\'re using WordPress version ' . get_bloginfo('version') . '. Please <a href="' . admin_url('update-core.php') . '" title="Update WP core">update</a>.</p></div>';
  } // min_version_error


  // add markup for UI overlay
  static function admin_footer() {
    if (wf_sn::is_plugin_page()) {
      echo '<div id="sn_overlay"><div class="sn-overlay-wrapper">';
      echo '<div class="inner">';

      // title
      echo '<div class="wf-sn-title">
             <h2><img src="' . WF_SN_PLUGIN_URL . 'images/security-ninja-pro-logo.png" alt="Security Ninja PRO" title="Security Ninja PRO"></h2>
           </div>';

      // Outer
      echo '<div class="wf-sn-overlay-outer">';

      // Content
      echo '<div class="wf-sn-overlay-content">';
      echo '<div id="sn-site-scan" style="display: none;">';
      echo '<h3>Security Ninja PRO is analyzing your site.<br/>It will only take a few moments ...</h3>';
      echo '</div>';

      do_action('sn_overlay_content');

      echo '<div class="loader"><img title="Loading ..." src="' . plugins_url('images/ajax-loader.gif', __FILE__) . '" alt="Loading..." /></div>';
      echo '<p><br><br><a id="abort-scan" href="#" class="button button-secondary input-button red">Stop</a></p>';

      do_action('sn_overlay_content_after');

      echo '</div>'; // wf-sn-overlay-content

      echo '</div></div></div></div>';

      echo '<div id="test-details-dialog" style="display: none;" title="Test details"><p>Please wait.</p></div>';

      echo '<div id="sn_tests_descriptions" style="display: none;">';
      require_once WF_SN_PLUGIN_DIR . 'sn-tests-description.php';
      echo '</div>';
    } // if is_plugin_page
  } // admin_footer


  // get info on new plugin version if one exists
  static function update_filter($current) {
    if (!self::is_activated()) {
      return $current;
    }

    static $response = false;
    $options = self::get_options();
    $plugin = 'security-ninja/security-ninja.php';

    if(empty($response) || is_wp_error($response)) {
      $request_params = array('sslverify' => false, 'timeout' => 15, 'redirection' => 2);
      $request_args = array('action' => 'update_info',
                            'timestamp' => time(),
                            'codebase' => 'pro',
                            'version' => self::$version,
                            'license_key' => $options['license_key'],
                            'site' => get_home_url());

      $url = add_query_arg($request_args, self::$licensing_servers[0]);
      $response = wp_remote_get(esc_url_raw($url), $request_params);

      if (is_wp_error($response)) {
        $url = add_query_arg($request_args, self::$licensing_servers[1]);
        $response = wp_remote_get(esc_url_raw($url), $request_params);
      }
    } // if !$response

    if (!is_wp_error($response) && wp_remote_retrieve_body($response)) {
      $data = json_decode(wp_remote_retrieve_body($response));
      if (empty($current)) {
        $current = new stdClass();
      }
      if (empty($current->response)) {
        $current->response = array();
      }
      if (!empty($data) && is_object($data)) {
        $data->icons = (array) $data->icons;
        $data->banners = (array) $data->banners;
        $current->response[$plugin] = $data;
      }
    }

    return $current;
  } // update_filter


  // get plugin info for lightbox
  static function update_details($result, $action, $args) {
    static $response = false;
    $options = self::get_options();
    $plugin = 'security-ninja';

    if ($action != 'plugin_information' || empty($args->slug) || ($args->slug != $plugin)) {
      return $result;
    }

    if(empty($response) || is_wp_error($response)) {
      $request_params = array('sslverify' => false, 'timeout' => 15, 'redirection' => 2);
      $request_args = array('action' => 'plugin_information',
                            'request_details' => serialize($args),
                            'timestamp' => time(),
                            'codebase' => 'pro',
                            'version' => self::$version,
                            'license_key' => $options['license_key'],
                            'site' => get_home_url());

      $url = add_query_arg($request_args, self::$licensing_servers[0]);
      $response = wp_remote_get(esc_url_raw($url), $request_params);

      if (is_wp_error($response) || !wp_remote_retrieve_body($response)) {
        $url = add_query_arg($request_args, self::$licensing_servers[1]);
        $response = wp_remote_get(esc_url_raw($url), $request_params);
      }
    } // if !$response

    if (is_wp_error($response) || !wp_remote_retrieve_body($response)) {
      $res = new WP_Error('plugins_api_failed', __('An unexpected HTTP error occurred during the API request.', 'google-maps-widget'));
    } else {
      $res = json_decode(wp_remote_retrieve_body($response), false);

      if (!is_object($res)) {
        $res = new WP_Error('plugins_api_failed', __('Invalid API respone.', 'google-maps-widget'), wp_remote_retrieve_body($response));
      } else {
        $res->sections = (array) $res->sections;
        $res->banners = (array) $res->banners;
        $res->icons = (array) $res->icons;
      }
    }

    return $res;
  } // update_details


  // ajax check activation code and save if valid
  static function activate_license_key_ajax() {
    check_ajax_referer('wf_sn_activate_license_key');
    
    $old_options = self::get_options();
    
    if (isset($_POST['license_key']) && $_POST['license_key'] === '') {
      $new_values['license_type'] = '';
      $new_values['license_expires'] = '';
      $new_values['license_active'] = false;
      $new_values['license_key'] = '';
      self::add_settings_error('License key saved!', 'updated');
      update_option(WF_SN_OPTIONS_KEY, array_merge($old_options, $new_values));
      wp_send_json_success();
    } else {
      $tmp = self::validate_license_key($_POST['license_key']);
      $new_values['license_key'] = $_POST['license_key'];
      if ($tmp['success']) {
        $new_values['license_type'] = $tmp['license_type'];
        $new_values['license_expires'] = $tmp['license_expires'];
        $new_values['license_active'] = $tmp['license_active'];
        if ($tmp['license_active'] == true) {
          self::add_settings_error('License key saved and activated!', 'updated');
          update_option(WF_SN_OPTIONS_KEY, array_merge($old_options, $new_values));
          wp_send_json_success();
        } else {
          self::add_settings_error('License not active. ' . $tmp['error'], 'error');
          update_option(WF_SN_OPTIONS_KEY, array_merge($old_options, $new_values));
          wp_send_json_success();
        }        
      } else {
        wp_send_json_error(array('error'=>$tmp['error'],'error_code'=>404));
      }
    }
    die();    
  } // activate_license_key_ajax

  static function save_license_key_ajax() {
    check_ajax_referer('wf_sn_save_license_key');

    $old_options = self::get_options();

    $new_values['license_type'] = $_POST['license_type'];
    $new_values['license_expires'] = $_POST['license_expires'];
    $new_values['license_active'] = $_POST['license_active']=='true'?true:false;
    $new_values['license_key'] = $_POST['license_key'];
    self::add_settings_error('License key saved and activated!', 'updated');
    
    update_option(WF_SN_OPTIONS_KEY, array_merge($old_options, $new_values));
    if ($_POST['license_active']) {
      wp_send_json_success();
    } else {
      wp_send_json_error(array('error'=>$tmp['error'],'error_code'=>$tmp['error_code']));
    }
  }


  // check if activation code is valid
  static function validate_license_key($code) {
    $request_params = array('sslverify' => false, 'timeout' => 25, 'redirection' => 2);
    $request_args = array('action' => 'validate_license',
                          'license_key' => $code,
                          'code_base' => 'pro',
                          'version' => self::$version,
                          'site' => get_home_url());

    $out = array('success' => false, 'license_active' => false, 'activation_code' => $code, 'error' => '', 'license_type' => '', 'license_expires' => '1900-01-01');

    $url = add_query_arg($request_args, self::$licensing_servers[0]);
    $response = wp_remote_get(esc_url_raw($url), $request_params);

    if (is_wp_error($response) || !wp_remote_retrieve_body($response)) {
      $url = add_query_arg($request_args, self::$licensing_servers[1]);
      $response = wp_remote_get(esc_url_raw($url), $request_params);
    }

    if (!is_wp_error($response) && wp_remote_retrieve_body($response)) {
      $result = wp_remote_retrieve_body($response);
      $result = json_decode($result, true);
      if (is_array($result['data']) && sizeof($result['data']) == 4) {
        $out['success'] = true;
        $out = array_merge($out, $result['data']);
      } else {
        $out['error'] = array('error'=>'Invalid response from licensing server. Please try again later.','error_code'=>1);
      }
    } else {
      $out['error'] = array('error'=>'Unable to contact licensing server. Please try again in a few moments.','error_code'=>404);
    }

    return $out;
  } // validate_license_key


  // check if license key is valid and not expired
  static function is_activated($license_type = false) {
    return true;
    $options = self::get_options();

    if (isset($options['license_active']) && $options['license_active'] === true &&
        isset($options['license_expires']) && $options['license_expires'] >= date('Y-m-d')) {

      if (mt_rand(0, 1000) > 998 && is_admin()) {
        $tmp = self::validate_license_key($options['license_key']);
        if ($tmp['success']) {
          $update['license_type'] = $tmp['license_type'];
          $update['license_expires'] = $tmp['license_expires'];
          $update['license_active'] = $tmp['license_active'];
          update_option(WF_SN_OPTIONS_KEY, array_merge($options, $update));
        }
      } // random license revalidation

      // check for specific license type?
      if (!empty($license_type)) {
        if (strtolower(trim($license_type)) == strtolower($options['license_type'])) {
          return true;
        } else {
          return false;
        }
      } // check specific license type

      return true;
    } else {
      return false;
    }
  } // is_activated


  // return default options
  static function default_options() {
    $defaults = array('license_key'      => '',
                      'license_active'    => false,
                      'license_expires'   => '',
                      'license_type'      => '',
                     );

    return $defaults;
  } // default_settings


  // get plugin's options
  static function get_options() {
    $options = get_option(WF_SN_OPTIONS_KEY, array());

    if (!is_array($options)) {
      $options = array();
    }
    $options = array_merge(self::default_options(), $options);

    return $options;
  } // get_options


  // all settings are saved in one option
  static function register_settings() {
    register_setting(WF_SN_OPTIONS_KEY, WF_SN_OPTIONS_KEY, array(__CLASS__, 'sanitize_settings'));
  } // register_settings


  // sanitize settings on save
  static function sanitize_settings($new_values) {
    $old_options = self::get_options();

    if (empty($_POST['submit-license'])) {
      return array_merge($old_options, $new_values);
    }

    if (isset($new_values['license_key']) && $new_values['license_key'] === '') {
      $new_values['license_type'] = '';
      $new_values['license_expires'] = '';
      $new_values['license_active'] = false;
      $new_values['license_key'] = '';
    } else {
      $tmp = self::validate_license_key($new_values['license_key']);
      $new_values['license_key'] = $new_values['license_key'];
      if ($tmp['success']) {
        $new_values['license_type'] = $tmp['license_type'];
        $new_values['license_expires'] = $tmp['license_expires'];
        $new_values['license_active'] = $tmp['license_active'];
        if ($tmp['license_active'] == true) {
          add_settings_error(WF_SN_OPTIONS_KEY, 'license_key', 'License key saved and activated!', 'updated');
        } else {
          add_settings_error(WF_SN_OPTIONS_KEY, 'license_key', 'License not active. ' . $tmp['error'], 'error');
        }
      } else {
        add_settings_error(WF_SN_OPTIONS_KEY, 'license_key', 'Unable to contact licensing server. Please try again in a few moments.', 'error');
      }
    }

    return array_merge($old_options, $new_values);
  } // sanitize_settings


  // license tab
  static function tab_license() {
    $options = self::get_options();

    echo '<div class="submit-test-container">';
    echo '<form method="post" action="options.php">';
    settings_fields(WF_SN_OPTIONS_KEY);

    echo '<label for="license_key" style="font-weight: bold; ">License Key: </label>
          <input class="regular-text" name="' . WF_SN_OPTIONS_KEY . '[license_key]" type="text" id="license_key" value="' . (!$options['license_hide']?esc_attr($options['license_key']):'') . '" placeholder="12345678-12345678-12345678-12345678"><br>';
    echo '<p class="description">';
    if (!self::is_activated()) {
      echo 'License key can be found in the confirmation email you received after purchasing.<br>';
      echo 'If you don\'t have a license - <a href="' . self::generate_sn_web_link('tab_license') . '" target="_blank">purchase one now</a>. If you\'re having problems activating - contact <a href="mailto:securityninja@webfactoryltd.com?subject=Security%20Ninja%20PRO%20problem">support</a>.';
    } else {
      echo 'If you\'re having problems please contact <a href="mailto:securityninja@webfactoryltd.com?subject=Security%20Ninja%20PRO%20problem">support</a>.';
    }
    echo '</p>';
    echo '<br>';
    echo '<p>';
    echo '<strong>License Status: </strong>';
    if (self::is_activated()) {
      if ($options['license_expires'] == '2035-01-01') {
        $valid = 'forever';
      } else {
        $valid = 'until ' . date('F jS, Y', strtotime($options['license_expires']));
        if (date('Y-m-d') == $options['license_expires']) {
          $valid .= '; expires today';
        } elseif (date('Y-m-d', time() + DAY_IN_SECONDS) == $options['license_expires']) {
          $valid .= '; expires tomorrow';
        } elseif (date('Y-m-d', time() + 30 * DAY_IN_SECONDS) > $options['license_expires']) {
          $tmp = (strtotime($options['license_expires'] . date(' G:i:s')) - time()) / DAY_IN_SECONDS;
          $valid .= '; expires in ' . round($tmp) . ' days';
        }
      }

      echo '<span style="color: green;">Active</span><br>
          <strong>Type: </strong>' . str_replace('pro', 'PRO', $options['license_type']);
      echo '<br><strong>Valid: </strong>' . $valid . '';
    } else {
      echo '<span style="color: red">Inactive</span>';
    }
    echo '</p>';
    echo '<br>';

    echo get_submit_button('Save and Validate License Key', 'primary large', 'submit-license', true, array());
    echo '</form>';
    echo '</div>';
  } // tab_license


  // helper function to generate tagged buy links
  static function generate_sn_web_link($placement = '', $page = '/', $params = array()) {
    $base_url = 'https://wpsecurityninja.com';
    if ('/' != $page) {
      $page = '/' . trim($page, '/') . '/';
    }
    $parts = array_merge(array('utm_source' => 'security_ninja_pro', 'utm_medium' => 'plugin', 'utm_content' => $placement, 'utm_campaign' => 'security_ninja_v' . self::$version), $params);

    $out = $base_url . $page . '?' . http_build_query($parts, '', '&amp;');

    return $out;
  } // generate_sn_web_link


  // whole options page
  static function main_page() {
    // redundant but doesn't hurt
    if (!current_user_can('administrator'))  {
      wp_die('You do not have sufficient permissions to access this page.');
    }

    $options = self::get_options();

    if(isset($_GET['hide-license-tab'])){
      $options['license_hide'] = true;
      update_option(WF_SN_OPTIONS_KEY, array_merge($options, array( 'license_hide'=> true )));
    } else if(isset($_GET['show-license-tab'])){
      $options['license_hide'] = false;
      update_option(WF_SN_OPTIONS_KEY, array_merge($options, array( 'license_hide'=> false )));
    }

    settings_errors();

    $hidden_tabs = get_transient('wf_sn_hidden_tabs');
    if (!$hidden_tabs) {
      $hidden_tabs = array();
    }
    $tabs = array();
    $tabs[] = array('id' => 'sn_tests', 'class' => '', 'label' => 'Security Tests', 'callback' => array(__CLASS__, 'tab_tests'));
    $tabs = apply_filters('sn_tabs', $tabs);
    
    if (self::is_activated()) {
      if(!$options['license_hide']){
        $tabs[] = array('id' => 'sn_license', 'class' => 'sn_license', 'label' => 'License', 'callback' => array(__CLASS__, 'tab_license'));
      }
    } else {
      $tabs = array(array('id' => 'sn_license', 'class' => 'sn_license', 'label' => 'License', 'callback' => array(__CLASS__, 'tab_license')));
    }

    echo '<div class="wrap">';
    echo '<div class="wf-sn-title">
           <h2><img src="' . WF_SN_PLUGIN_URL . 'images/security-ninja-pro-logo.png" alt="Security Ninja PRO" title="Security Ninja PRO"></h2>
         </div>';

    echo '<div id="tabs">';
    echo '<ul>';
    foreach ($tabs as $tab) {
	    if (!empty($tab['label'])) {
      	  echo '<li><a href="#' . $tab['id'] . '" class="' . $tab['class'] . '">' . $tab['label'] . '</a></li>';
	    }
    } // foreach
    echo '</ul>';

    foreach ($tabs as $tab) {
      if (!empty($tab['callback'])) {
	      echo '<div style="display: none;" id="' . $tab['id'] . '">';
        call_user_func($tab['callback']);
		    echo '</div>';
	    }
    } // foreach

    echo '</div>'; // tabs
    echo '</div>'; // wrap
  } // main_page


  // display tests table
  static function tab_tests() {
    // get test results from cache
	  $tests = get_option(WF_SN_RESULTS_KEY);

    echo '<div class="submit-test-container">
          <input type="submit" value=" Analyze Site " id="run-tests" class="button-primary" name="Submit" />';

    if (!empty($tests['last_run'])) {
      echo '<span class="sn-notice">Tests were last run on ' . date(get_option('date_format') . ' @ ' . get_option('time_format'), $tests['last_run']) . '.';
      if (!empty($tests['run_time'])) {
        echo ' It took ' . number_format($tests['run_time'], 1) . ' seconds to run them.';
      }
      echo '</span>';

      $bad = $warning = $good = $score = $total =  0;
      foreach($tests['test'] as $test_details) {
        $total += $test_details['score'];
        if ($test_details['status'] == 10) {
          $good++;
          $score += $test_details['score'];
        } elseif ($test_details['status'] == 0) {
          $bad++;
        } else {
          $warning++;
        }
      }

      $score = round($score / $total * 100);

      echo '<div id="counters">';
      if ($good == 1) {
        echo '<span class="good">' . $good . '<br><i>test passed</i></span>';
      } else {
        echo '<span class="good">' . $good . '<br><i>tests passed</i></span>';
      }
      if ($warning == 1) {
        echo '<span class="warning">' . $warning .'<br><i>test has warnings</i></span>';
      } else {
        echo '<span class="warning">' . $warning .'<br><i>tests have warnings</i></span>';
      }
      if ($bad == 1) {
        echo '<span class="bad">' . $bad . '<br><i>test failed</i></span>';
      } else {
        echo '<span class="bad">' . $bad . '<br><i>tests failed</i></span>';
      }
      echo '<span class="score">' . $score . '%<br><i>overall site score</i></span>';
      echo '</div>';
    }

    if (empty($tests['last_run'])) {
      echo '<p>Security Ninja PRO <b>tests were never run</b>. Click "Analyze Site" to run them now and analyze your site for security vulnerabilities.</p>';
    }
    echo '<p><strong>Please read!</strong> These tests only serve as suggestions! Although they cover years of best practices in security getting all test <i>green</i> does not guarantee your site will not get hacked. Likewise, having them all <i>red</i> doesn\'t mean you\'ll certainly get hacked. Please read each test\'s detailed information to see if it represents a real security issue for your site. Suggestions and test results apply to public, production sites, not local, development ones.</p>';

    echo '</div>';

    $out = '';
    if (!empty($tests['last_run'])) {

      $out .= '<table class="wp-list-table widefat" cellspacing="0" id="security-ninja">';
      $out .= '<thead><tr>';
      $out .= '<th class="sn-status">Status</th>';
      $out .= '<th>Test description</th>';
      $out .= '<th>Test results</th>';
      $out .= '<th>&nbsp;</th>';
      $out .= '</tr></thead>';
      $out .= '<tbody>';

      if (is_array($tests['test'])) {
        // test Results
        foreach($tests['test'] as $test_name => $details) {
          if (substr($test_name, 0, 3) == 'ad_' || $test_name[0] == '_') {
            continue;
          }

          $out .= '<tr>
                  <td class="sn-status">' . wf_sn::status($details['status']) . '</td>
                  <td>' . $details['title'] . '</td>
                  <td>' . $details['msg'] . '</td>';

				  if (class_exists('wf_sn_af_fix_' . $test_name) && $details['status'] != 10) {
					  $details_label = 'Details, Tips &amp; Auto Fixer';
				  } else {
					  $details_label = 'Details &amp; Tips';
				  }
				  $out .= '<td class="sn-details"><a data-test-status="' . $details['status'] . '" data-test-id="' . $test_name . '" href="#' . $test_name . '" class="button action">' . $details_label . '</a></td>';
          $out .= '</tr>';
        } // foreach ($tests)
      } else { // no test results
        $out .= '<tr>
                <td colspan="4">No test results are available. Click "Analyze Site" to run tests now.</td>
              </tr>';
      } // if tests

      $out .= '</tbody>';
      $out .= '<tfoot><tr>';
      $out .= '<th class="sn-status">Status</th>';
      $out .= '<th>Test description</th>';
      $out .= '<th>Test results</th>';
      $out .= '<th>&nbsp;</th>';
      $out .= '</tr></tfoot>';
      $out .= '</table>';
    } // if $results

    $out = apply_filters('sn_tests_table', $out, $tests);
    echo $out;
  } // tab_tests


  // run all tests; via AJAX
  static function run_tests($return = false) {
    if (defined('DOING_AJAX') && DOING_AJAX) {
      check_ajax_referer('wf_sn_run_tests');
    }

    @set_time_limit(WF_SN_MAX_EXEC_SEC);
    $test_count = 0;
    $start_time = microtime(true);
    $test_description['last_run'] = current_time('timestamp');

    foreach(wf_sn_tests::$security_tests as $test_name => $test){
      if ($test_name[0] == '_' || in_array($test_name, self::$skip_tests) || substr($test_name, 0, 3) == 'ad_') {
        continue;
      }
      $response = wf_sn_tests::$test_name();

      $test_description['test'][$test_name]['title'] = $test['title'];
      $test_description['test'][$test_name]['status'] = $response['status'];
      $test_description['test'][$test_name]['score'] = $test['score'];

      if (!isset($response['msg'])) {
        $response['msg'] = '';
      }

      if ($response['status'] == 10) {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_ok'], $response['msg']);
      } elseif ($response['status'] == 0) {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_bad'], $response['msg']);
      } else {
        $test_description['test'][$test_name]['msg'] = sprintf($test['msg_warning'], $response['msg']);
      }
      $test_count++;
    } // foreach

    $test_description['run_time'] = microtime(true) - $start_time;

    do_action('security_ninja_done_testing', $test_description, microtime(true) - $start_time);

    $pointers = get_option(WF_SN_POINTERS_KEY);
    if (!empty($pointers['welcome'])) {
      unset($pointers['welcome']);
      update_option(WF_SN_POINTERS_KEY, $pointers);
    }


    if ($return) {
      return $test_description;
    } else {
      update_option(WF_SN_RESULTS_KEY, $test_description);
      die('1');
    }
  } // run_test


  // convert status integer to button
  static function status($int) {
    if ($int == 0) {
      $string = '<span class="sn-error">Failed</span>';
    } elseif ($int == 10) {
      $string = '<span class="sn-success">Passed</span>';
    } else {
      $string = '<span class="sn-warning">Warning</span>';
    }

    return $string;
  } // status


  // reset pointers on activation and save some info
  static function activate() {
    $pointers = array();
    $pointers['welcome'] = array('target' => '#menu-tools', 'edge' => 'left', 'align' => 'right', 'content' => 'Thank you for installing <b>Security Ninja PRO</b>! Please open <a href="tools.php?page=wf-sn">Tools - Security Ninja PRO</a> to analyze &amp; protect your site.');
    update_option(WF_SN_POINTERS_KEY, $pointers);

    $options = self::get_options();
    if (empty($options['first_version']) || empty($options['first_install'])) {
      $options['first_version'] = wf_sn::get_plugin_version();
      $options['first_install'] = current_time('timestamp');
      update_option(WF_SN_OPTIONS_KEY, $options);
    }
  } // activate


  // clean-up when deactivated
  static function deactivate() {
    delete_option(WF_SN_RESULTS_KEY);
    delete_option(WF_SN_POINTERS_KEY);
    delete_transient('wf_sn_hidden_tabs');
  } // deactivate


  // clean-up when uninstalled
  static function uninstall() {
    delete_option(WF_SN_OPTIONS_KEY);
    delete_option(WF_SN_POINTERS_KEY);
    delete_transient('wf_sn_hidden_tabs');
  } // uninstall


  // activate with license key provided by Marketplace
  static function activate_from_marketplace($params) {
    $options = self::get_options();
    $tmp = self::validate_license_key($params['license_key']);
    $options['license_key'] = $params['license_key'];

    if ($tmp['success']) {
      $options['license_type'] = $tmp['license_type'];
      $options['license_expires'] = $tmp['license_expires'];
      $options['license_active'] = $tmp['license_active'];
      if ($tmp['license_active'] == true) {
        $out = true;
      } else {
        $out = new WP_error(-1, 'Security Ninja license not active. ' . $tmp['error']);
      }
    } else {
      $out = new WP_error(-1, 'Unable to contact Security Ninja licensing server. Please try again in a few moments.');
    }

    update_option(WF_SN_OPTIONS_KEY, $options);
    return $out;
  } // activate_from_marketplace
} // wf_sn class


// hook everything up
register_activation_hook(__FILE__, array('WF_SN', 'activate'));
register_deactivation_hook(__FILE__, array('WF_SN', 'deactivate'));
register_uninstall_hook(__FILE__, array('WF_SN', 'uninstall'));
add_action('init', array('WF_SN', 'init'));
add_action('plugins_loaded', array('WF_SN', 'plugins_loaded'));
