/*
 * Security Ninja - Malware Scanner add-on
 * (c) 2015. Web factory Ltd
 */

;jQuery(document).ready(function($){
  // toggle file info details
  $('#sn_ms_results').on('click', '.toggle-slide', function(e){
    e.preventDefault();

    content = $(this).parents('.sn-malware-filebox').find('.sn-malware-file-code');

    $(this).toggleClass('opened');
    $(content).slideToggle(500);
  });

  // reset files whitelist
  $('#sn-ms-reset-whitelist').on('click', function(e){
    e.preventDefault();

    if (!confirm('Are you sure you want to reset the whitelisted files list?\nPlease rescan files after cleaning the whitelist.')) {
      return;
    }
    $.post(ajaxurl, {_ajax_nonce: wf_sn_ms_nonce, action: 'sn_ms_reset_whitelist'}, function(response) {
      window.location.reload();
    });
  }); // reset files whitelist

  // whitelist file
  $('#sn_ms_results').on('click', 'button.sn_ms_whitelist', function(e){
    e.preventDefault();

    tmp = confirm('Are you sure you have checked this file and want to whitelist it?');
    if (!tmp) {
      return;
    }

    $(this).parents('.sn-malware-filebox').slideUp(500);

    $.post(ajaxurl,
           {action: 'sn_ms_whitelist_file', hash: $(this).data('hash'), _ajax_nonce: wf_sn_ms_nonce},
           function(response){ },
           'json'
           );
  }); // whitelist files

  // delete file
  $('#sn_ms_results').on('click', 'button.sn_ms_delete', function(e){
    e.preventDefault();

    tmp = confirm('Are you sure you want to delete this file? There is NO undo!\n' + $(this).data('filename'));
    if (!tmp) {
      return;
    }

    button = this;

    $.post(ajaxurl,
           {action: 'sn_ms_delete_file', filename: $(this).data('filename'), hash: $(this).data('hash'), _ajax_nonce: wf_sn_ms_nonce},
           function(response){
             if (typeof response.success == 'undefined') {
               alert('Undocumented error. File was not deleted. Please reload the page.');
             } else if (response.success == true) {
               alert('File deleted!');
               $(button).parents('.sn-malware-filebox').slideUp(500);
             } else {
               alert('Your file access properties do not allow us to delete the file (this is not a bad thing). Please delete it manually, via FTP.');
             }
           },
           'json'
           );
  }); // whitelist files

  // run test
  $('#sn_ms_run_scan').on('click', function(e) {
    e.preventDefault();

    startingpoint = $.Deferred();
    startingpoint.resolve();

    $('#sn_ms_results').html('');
    $('ul.malware-scan-list').html('');
    $('.wrap .error').hide();

    $.each(actions, function(i, action) {
      $('ul.malware-scan-list').append('<li id="msg_' + action.action +'" class="pending">' + action.msg1 + ' <span class="badge">Waiting ...</span></li>');
    });
    $('.loader').hide();

    sn_block_ui('#sn-malware-scanner');

    $.each(actions, function(i, action) {
      startingpoint = startingpoint.pipe( function() {
        $('#msg_' + action.action).removeClass('pending').addClass('current');
        $('.badge', '#msg_' + action.action).html('Working ...');
        return do_sn_ms_request(action);
      });
    }); // each action
    
    $('.wf-sn-ms-not-run').hide();
  }); // run tests

  // handle each individual ajax call
  function do_sn_ms_request(params) {
    return $.ajax({ type: 'POST',
      url: ajaxurl,
      data: {action: 'sn_ms_run_scan', what: params.action, _ajax_nonce: wf_sn_ms_nonce},
      dataType: 'json',
      success: function(response) {
        if (!response.success) {
          alert('Invalid option. Please reload');
          return;
        }

        $('#msg_' + params.action).removeClass('pending').removeClass('current').addClass('passed');
        $('.badge', '#msg_' + params.action).html(params.msg2.replace('%d', parseInt(response.data.cnt, 10)));

        if (params.action == 'get_results') {
          $('#sn_ms_results').append(response.data);
          sn_unblock_ui('#sn-malware-scanner');
          $('.loader').show();
        }
      }, // success
      error: function(response){
        alert('An undocumented error has occured. Please reload the page and try again.');
      } // error
    });
  } // do_sn_ms_request

var actions = [{msg1: 'Cleaning cache ... ', msg2: ' DONE!', action: 'clean_cache'},

               {msg1: 'Generating list of wp_options table rows ... ', msg2: 'DONE! Found %d rows.', action: 'get_db_rows'},
               {msg1: 'Generating list of theme folders ... ', msg2: 'DONE! Found %d folders.', action: 'get_theme_folders'},
               {msg1: 'Generating list of plugin folders ... ', msg2: 'DONE! Found %d folders.', action: 'get_plugin_folders'},
               {msg1: 'Generating list of wp-content folders ... ', msg2: 'DONE! Found %d folders.', action: 'get_content_folders'},

               {msg1: 'Generating list of theme files ... ', msg2: 'DONE! Found %d files.', action: 'get_theme_files'},
               {msg1: 'Generating list of plugin files ... ', msg2: 'DONE! Found %d files.', action: 'get_plugin_files'},
               {msg1: 'Generating list of wp-content files ... ', msg2: 'DONE! Found %d files.', action: 'get_content_files'},

               {msg1: 'Scanning wp_options table ... ', msg2: 'DONE! Found %d suspicious rows.', action: 'scan_db'},
               {msg1: 'Scanning theme files ... ', msg2: 'DONE! Found %d suspicious files.', action: 'scan_theme_files'},
               {msg1: 'Scanning plugin files ... ', msg2: 'DONE! Found %d suspicious files.', action: 'scan_plugin_files'},
               {msg1: 'Scanning wp-content files ... ', msg2: 'DONE! Found %d suspicious files.', action: 'scan_content_files'},

               {msg1: 'Generating results ... ', msg2: 'DONE!', action: 'get_results'}];

}); // onload