<?php
if (!function_exists('add_action')) {
  die('Please don\'t open this file directly!');
}

define('WF_SN_MS_OPTIONS', 'wf_sn_ms_options');
define('WF_SN_MS_CACHE', 'wf_sn_ms_cache');
define('WF_SN_MS_RESULTS', 'wf_sn_ms_results');
define('WF_SN_MS_WHITELIST', 'wf_sn_ms_whitelist');
define('WF_SN_MS_DELETELIST', 'wf_sn_ms_deletelist');


class wf_sn_ms {
  // init plugin
  static function init() {
    // does the user have enough privilages to use the plugin?
    if (is_admin()) {
      // add tab to Security Ninja tabs
      add_filter('sn_tabs', array(__CLASS__, 'sn_tabs'));

      // enqueue scripts
      add_action('admin_enqueue_scripts', array(__CLASS__, 'enqueue_scripts'));

      // register ajax endpoints
      add_action('wp_ajax_sn_ms_run_scan', array(__CLASS__, 'run_scan'));
      add_action('wp_ajax_sn_ms_whitelist_file', array(__CLASS__, 'add_whitelisted_file'));
      add_action('wp_ajax_sn_ms_reset_whitelist', array(__CLASS__, 'reset_whitelist'));
      add_action('wp_ajax_sn_ms_delete_file', array(__CLASS__, 'delete_file'));
      // warn if tests were never run
      add_action('admin_notices', array(__CLASS__, 'warnings'));

      // add custom text for GUI overlay
      add_action('sn_overlay_content', array(__CLASS__, 'overlay_content'));
    } // if admin
  } // init

  
  // enqueue CSS and JS scripts on plugin's admin page
  static function enqueue_scripts() {
    if (wf_sn::is_plugin_page()) {
      $plugin_url = plugin_dir_url(__FILE__);

      wp_enqueue_script('sn-ms-js', $plugin_url . 'js/wf-sn-ms.js', array('jquery'), wf_sn::$version, true);
      wp_enqueue_style('sn-ms-css', $plugin_url . 'css/wf-sn-ms.css', array(), wf_sn::$version);

      wp_localize_script('jquery', 'wf_sn_ms_nonce', wp_create_nonce('sn_ms'));
    } // if
  } // enqueue_scripts


  // add new tab
  static function sn_tabs($tabs) {
    $core_tab = array('id' => 'sn_malware', 'class' => '', 'label' => 'Malware Scanner', 'callback' => array(__CLASS__, 'malware_page'));
    $done = 0;

    for ($i = 0; $i < sizeof($tabs); $i++) {
      if ($tabs[$i]['id'] == 'sn_malware') {
        $tabs[$i] = $core_tab;
        $done = 1;
        break;
      }
    } // for

    if (!$done) {
      $tabs[] = $core_tab;
    }

    return $tabs;
  } // sn_tabs

  
  // add custom message to overlay
  static function overlay_content() {
    echo '<div id="sn-malware-scanner" style="display: none;">';
    echo '<h3>Security Ninja Malware Scanner is scanning your files.<br/>It will only take a few moments ...</h3>';
    echo '<ul class="malware-scan-list"></ul>';
    echo '</div>';
  } // overlay_content


  // generate a list of files to scan in a folder
  static function scan_folder($path, $extensions = null, $depth = 3, $relative_path = '' ) {
    if (!is_dir($path))
      return false;

    if ($extensions) {
      $extensions = (array) $extensions;
      $_extensions = implode('|', $extensions);
    } else {
      $extensions = array('php', 'php3', 'inc');
      $_extensions = implode('|', $extensions);
    }

    $relative_path = trailingslashit($relative_path);
    if ('/' == $relative_path) {
      $relative_path = '';
    }

    $results = scandir($path);
    $files = array();

    foreach ($results as $result) {
      if ('.' == $result[0]) {
        continue;
      }

      if (is_dir($path . '/' . $result)) {
        if (!$depth || 'CVS' == $result ) {
          continue;
        }
        $found = self::scan_folder($path . '/' . $result, $extensions, $depth - 1 , $relative_path . $result);
        $files = array_merge_recursive($files, $found);
      } elseif (!$extensions || preg_match('~\.(' . $_extensions . ')$~', $result)) {
        $files[$relative_path . $result] = $path . '/' . $result;
      }
    } // foreach result

    return $files;
  } // scan_folder


  // generate a list of folders within a folder to scan
  static function get_folders($parent_folder, $skip = null) {
    if (!$parent_folder) {
      return array();
    }

    $folders = glob(trailingslashit($parent_folder) . '*' , GLOB_ONLYDIR);

    if (isset($skip) && sizeof($folders)) {
      $tmp = sizeof($folders);
      $cleaned = array();
      $skip = (array) $skip;
      for ($i=0; $i < $tmp; $i++) {
        $filtered = false;
        foreach ($skip as $pattern) {
          if (stripos($folders[$i], $pattern) !== false) {
            $filtered = true;
          }
        } // foreach skip
        if (!$filtered) {
          $cleaned[] = $folders[$i];
        }
      } // for folders
      $folders = $cleaned;
    } // if skip

    if (!$folders) {
      return array();
    } else {
      return $folders;
    }
  } // get_folders


  // save cache data
  static function update_cache($data, $key = null) {
    if (!isset($key)) {
      $cache = $data;
    } else {
      $cache = get_option(WF_SN_MS_CACHE);
      $cache[$key] = $data;
    }

    return update_option(WF_SN_MS_CACHE, $cache);
  } // update_cache


  // retreive cached data
  static function get_cache($key = null) {
    $cache = get_option(WF_SN_MS_CACHE);

    if (isset($key) && isset($cache[$key])) {
      return $cache[$key];
    } elseif (isset($key)) {
      return array();
    } else {
      return $cache;
    }
  } // get_cache


  // save results data
  static function update_results($data, $key = null) {
    if (!isset($key)) {
      $cache = $data;
    } else {
      $cache = get_option(WF_SN_MS_RESULTS);
      $cache[$key] = $data;
    }
    $cache['last_run'] = current_time('timestamp');

    return update_option(WF_SN_MS_RESULTS, $cache);
  } // update_results


  // retreive results data
  static function get_results($key = null) {
    $cache = get_option(WF_SN_MS_RESULTS);

    if (isset($key) && isset($cache[$key])) {
      return $cache[$key];
    } elseif (isset($key)) {
      return array();
    } else {
      return $cache;
    }
  } // get_cache


  // add file hash to whitelist
  static function add_whitelisted_file() {
    check_ajax_referer('sn_ms');

    $hash = trim(@$_POST['hash']);
    $whitelist = get_option(WF_SN_MS_WHITELIST, array());
    $whitelist[] = $hash;
    $whitelist = array_unique($whitelist);
    update_option(WF_SN_MS_WHITELIST, $whitelist);

    wp_send_json_success();
  } // add_whitelisted_file


  // remove all files from user created whitelist
  static function reset_whitelist() {
    check_ajax_referer('sn_ms');

    update_option(WF_SN_MS_WHITELIST, array());

    wp_send_json_success();
  } // reset_whitelist


  // delete file via ajax
  static function delete_file() {
    check_ajax_referer('sn_ms');

    $file = @$_POST['filename'];
    $hash = trim(@$_POST['hash']);

    if (filter_var($file, FILTER_SANITIZE_URL) !== $file) {
      die("-1");
    }

    if ($hash != @md5_file($file)) {
      die("-1");
    }

    if (@unlink($file)) {
      $deletelist = get_option(WF_SN_MS_DELETELIST, array());
      $deletelist[] = $hash;
      $deletelist = array_unique($deletelist);
      update_option(WF_SN_MS_DELETELIST, $deletelist);
      wp_send_json_success();
    } else {
      wp_send_json_error();
    }
  } // delete_file


  // ajax function for running scan
  static function run_scan($opt = false) {
    if (defined('DOING_AJAX') && DOING_AJAX) {
      check_ajax_referer('sn_ms');
    }

    $out['err'] = '';
    if (!$opt) {
      $opt = trim(@$_POST['what']);
    }

    switch ($opt) {
      case 'clean_cache':
        self::update_cache(array());
        self::update_results(array());
        delete_option(WF_SN_MS_DELETELIST);
      break;
      case 'get_db_rows':
        global $wpdb;

        $query = $wpdb->prepare('SELECT COUNT(option_id) FROM ' . $wpdb->options . ' WHERE option_name NOT LIKE %s', '\_%');
        $cnt = $wpdb->get_var($query);

        $out['cnt'] = $cnt;
      break;
      case 'get_plugin_folders':
        $folders = self::get_folders(WP_PLUGIN_DIR);
        self::update_cache($folders, 'plugin_folders');
        $out['cnt'] = sizeof($folders);
      break;
      case 'get_theme_folders':
        global $wp_theme_directories;
        $folders = self::get_folders($wp_theme_directories[0]);
        self::update_cache($folders, 'theme_folders');
        $out['cnt'] = sizeof($folders);
      break;
      case 'get_content_folders':
        $folders = self::get_folders(WP_CONTENT_DIR, array('themes', 'plugins'));
        self::update_cache($folders, 'content_folders');
        $out['cnt'] = sizeof($folders);
      break;
      case 'get_theme_files':
        $folders = self::get_cache('theme_folders');
        $tmp = 0;
        $all_files = array();
        foreach ($folders as $folder) {
          $files = self::scan_folder($folder);
          $all_files = array_merge($all_files, $files);
          $tmp += sizeof($files);
        }
        self::update_cache($all_files, 'theme_files');
        $out['cnt'] = $tmp;
      break;
      case 'get_plugin_files':
        $folders = self::get_cache('plugin_folders');
        $tmp = 0;
        $all_files = array();
        foreach ($folders as $folder) {
          $files = self::scan_folder($folder);
          $all_files = array_merge($all_files, $files);
          $tmp += sizeof($files);
        }
        self::update_cache($all_files, 'plugin_files');
        $out['cnt'] = $tmp;
      break;
      case 'get_content_files':
        $folders = self::get_cache('content_folders');
        $tmp = 0;
        $all_files = array();
        foreach ($folders as $folder) {
          $files = self::scan_folder($folder);
          $all_files = array_merge($all_files, $files);
          $tmp += sizeof($files);
        }
        self::update_cache($all_files, 'content_files');
        $out['cnt'] = $tmp;
      break;
      break;
        case 'scan_db':
        $results = self::scan_db();
        self::update_results($results, 'db_options');
        $out['cnt'] = sizeof($results);
      break;
      case 'scan_theme_files':
        $files = self::get_cache('theme_files');
        $results = self::process_files($files);
        self::update_results($results, 'theme_files');
        $out['cnt'] = sizeof($results);
      break;
      case 'scan_plugin_files':
        $files = self::get_cache('plugin_files');
        $results = self::process_files($files);
        self::update_results($results, 'plugin_files');
        $out['cnt'] = sizeof($results);
      break;
      case 'scan_content_files':
        $files = self::get_cache('content_files');
        $results = self::process_files($files);
        self::update_results($results, 'content_files');
        $out['cnt'] = sizeof($results);
      break;
      case 'get_results':
        $out = '';
        $cnt = 0;

        $files = self::get_results('db_options');
        $out .= self::format_db_results($files);

        $files = self::get_results('theme_files');
        $out .= self::format_results($files);
        $cnt += sizeof($files);

        $files = self::get_results('plugin_files');
        $out .= self::format_results($files);
        $cnt += sizeof($files);

        $files = self::get_results('content_files');
        $out .= self::format_results($files);
        $cnt += sizeof($files);

        if (defined('DOING_AJAX') && DOING_AJAX) {
          do_action('security_ninja_malware_scanner_done_scanning', $cnt);
        }

        if ($out) {
          $out = '<h3 class="files-list-desc">Content of the following files matched some of our malware detection rules. This does not automatically make a file malicious but it does require a closer inspection.</h3>' . $out;
        } else {
          $out .= '<h3 class="files-list-desc">Great! There are no suspicious files to report at the moment.</h3>';
        }
      break;
      default:
        wp_send_json_error('Invalid option.');
    }

    if (defined('DOING_AJAX') && DOING_AJAX) {
      wp_send_json_success($out);
    } else {
      return $out;
    }
  } // run_scan


  // scan provided list of files, mind the whitelist
  static function process_files($files) {
    $tmp = 0;
    $results = array();
    $files = array_slice($files, 0, 500);

    foreach ($files as $file) {
      $hash = md5_file($file);
      if (self::in_whitelist($file, $hash)) {
        continue;
      }
      $result = self::scan_file($file);
      if ($result) {
        $results[] = array('filename' => $file, 'hash' => $hash, 'lines' => $result);
      }
    } //foreach file

    return $results;
  } // process_files


  // check if file is in whitelist, user or global one
  static function in_whitelist($filename, $hash) {
    $whitelist = get_option(WF_SN_MS_WHITELIST, array());
    $deletelist = get_option(WF_SN_MS_DELETELIST, array());
    $internal_whitelist = array('*/security-ninja*', '*/login-ninja/*', '*/5sec-google-authenticator/*', '*/google-maps-widget/*', '*/easy-digital-downloads/*', '*/optin-ninja/*', '*/smartmonitor/*', '*/under-construction-page/*', '*/profit_builder/*', '*/leadsflow-pro/*');

    if (in_array($hash, $whitelist) !== false) {
      return true;
    }

    if (in_array($hash, $deletelist) !== false) {
      return true;
    }

    foreach ($internal_whitelist as $pattern) {
      if (fnmatch($pattern, strtolower($filename))) {
        return true;
      }
    } // foreach internal whitelist

    return false;
  } // check_whitelist


  // prepare results for output
  static function format_results($files) {
    $out = '';

    if (!$files) {
      return '';
    }

    foreach ($files as $file) {
      if (self::in_whitelist($file['filename'], $file['hash'])) {
        continue;
      }
      $out .= '<div class="sn-malware-filebox">';
      $out .= '<div class="sn-malware-title">';
      $out .= '<div class="sn-malware-title-inner">';
      $out .= '<b>' . str_replace(ABSPATH, '', $file['filename']) . '</b> <a href="#" class="toggle-slide"><div class="dashicons-before dashicons-arrow-up-alt2"></div></a> <button class="input-button red sn_ms_delete" data-filename="' . $file['filename'] . '" data-hash="' . $file['hash'] . '">delete file</button> <button class="input-button gray sn_ms_whitelist" data-hash="' . $file['hash'] . '">whitelist file</button><br>';
      $out .= '</div>';
      $out .= '</div>';
      $out .= '<div class="sn-malware-file-code closed">';
      foreach ($file['lines'] as $nb => $line) {
        $tmp = htmlspecialchars($line[0]);
        $tmp = str_replace(array('@span@', '@/span@'), array('<span class="sn_ms_highlight">', '</span>'), $tmp);
        $out .= '<div class="one-line">';
        $out .= '<span class="line-number">' . $nb . '.</span> <p>' . $tmp . '</p>';
        $out .= '</div>';
      } // foreach line
      $out .= '</div>';
      $out .= '</div>';
    } // foreach $file

    return $out;
  } // format_results


  // prepare results for output
  static function format_db_results($options) {
    global $wpdb;
    $out = '';

    if (!$options) {
      return '';
    }

    $out .= '<div class="sn-malware-filebox">';
    $out .= '<div class="sn-malware-title">';
    $out .= '<div class="sn-malware-title-inner">';
    $out .= '<b>DB table ' . DB_NAME . '.' . $wpdb->options  . '</b> <a href="#" class="toggle-slide"><div class="dashicons-before dashicons-arrow-up-alt2"></div></a><br>';
    $out .= '</div>';
    $out .= '</div>';
    $out .= '<div class="sn-malware-file-code closed">';
    foreach ($options as $key => $result) {
        $result = htmlspecialchars($result);
        $result = str_replace(array('@span@', '@/span@'), array('<span class="sn_ms_highlight">', '</span>'), $result);
        $out .= '<div class="one-line">';
        $out .= '<span class="line-number">' . $key . '</span> <p>' . $result . '</p>';
        $out .= '</div>';
      } // foreach line
    $out .= '</div>';
    $out .= '</div>';

    return $out;
  } // format_db_results


  // generate regular expression for malware search
  static function line_match_pattern($filetype = 'php') {
    $php = '/(assert|file_get_contents|curl_exec|popen|proc_open|unserialize|eval|base64_encode|base64_decode|create_function|exec|shell_exec|system|passthru|ob_get_contents|file|curl_init|readfile|fopen|fsockopen|pfsockopen|fclose|fread|file_put_contents)\s*?\(/';

    return $php;
  } // line_match_pattern


  // scan individual file
  static function scan_file($file) {
    if (filter_var($file, FILTER_SANITIZE_URL) !== $file || !$file || !$content = file($file)) {
      return false;
    }

    $results = array();

    foreach($content as $num => $line) {
      if ($result = self::scan_file_line($line, $num)) {
        $results[$num] = $result;
      }
    }

    if (!empty($results)) {
      return $results;
    }

    return false;
  } // scan_file


  // scans all option and transient strings in db.options
  static function scan_db() {
    global $wpdb;
    $out = array();

    $query = $wpdb->prepare('SELECT COUNT(option_id) FROM ' . $wpdb->options . ' WHERE option_name NOT LIKE %s', '\_%');
    $records = $wpdb->get_var($query);

    $pages = round($records / 100) + 1;
    $pages = max(13, $pages);
    for ($i = 0; $i < $pages; $i++) {
      $query = $wpdb->prepare('SELECT option_name, option_value FROM ' . $wpdb->options . ' WHERE option_name  NOT LIKE %s LIMIT %d, 100', '\_%', $i * 100);
      $query = $wpdb->get_results($query);
      foreach ($query as $row) {
        $result = self::scan_file_line($row->option_value, 0, true);
        if ($result) {
          $out[$row->option_name] = $result[0];
        }
      }
      unset($query);
      wp_cache_flush();
    } // for

    return $out;
  } // scan_db_options


  // scan one file line
  static function scan_file_line($line, $num, $only_first_line = false) {
    $line = trim((string) $line);

    if (!$line) {
      return false;
    }

    $results = array();
    $output = array();

    preg_match_all(self::line_match_pattern('php'), $line, $matches);
    if ($matches[1]) {
      $results = $matches[1];
    }

    preg_match_all('/[\'\"\$\\ \/]*?([a-zA-Z0-9]{' .strlen(base64_encode('rainbow + unicorn = magic.')). ',})/', $line,$matches);
    if ($matches[1]) {
      $results = array_merge($results, $matches[1]);
    }

    preg_match_all('/<\s*?(i?frame)/', $line, $matches);
    if ($matches[1]) {
      $results = array_merge($results, $matches[1]);
    }

    /* MailPoet Vulnerability */
    preg_match_all('/explode\s?\(chr\s?\(\s?\(\d{3}\s?-\s?\d{3}\s?\)\s?\)\s?,/', $line, $matches);
    if ($matches[0]) {
      $results = array_merge($results, $matches[0]);
    }

    if ($results) {
      $results = array_unique($results);

      foreach($results as $tag) {
        $string = str_replace($tag, '@span@' . $tag . '@/span@', self::cut_line($line, $tag));
        $output[] = $string;
        if ($only_first_line) {
          break;
        }
      } // foreach result

      return $output;
    } // if results

    return false;
  } // scan_file_line


  // trim code line
  static function cut_line($line, $tag, $max = 200) {
    if (!$line || !$tag) {
      return false;
    }

    if (strlen($tag) > $max) {
      return $tag;
    }

    $left = round(($max - strlen($tag)) / 2);
    $tag = preg_quote($tag);

    $output = preg_replace('/(' . $tag . ')(.{' .$left. '}).{0,}$/', '$1$2 ...', $line);
    $output = preg_replace('/^.{0,}(.{' . $left . ',})(' . $tag . ')/', '... $1$2', $output);

    return $output;
  } // cut_line


  // display results
  static function malware_page() {
    $last_run = self::get_results('last_run');

    echo '<div class="submit-test-container">';
    echo '<a id="sn_ms_run_scan" href="#" class="input-button button-primary">Scan files for malware</a>';

    if (isset($last_run) && $last_run) {
      echo '<span class="sn-notice">Files were last scanned on: ' . date(get_option('date_format') . ' ' . get_option('time_format'), $last_run) . '.</span>';
    }

    echo '<p><strong>Please read!</strong> Files are scanned using a heuristic analysis method that compares their content to patterns and code samples often used by in malicious scripts. It is very important to understand that a file marked by Malware Scanner as suspicious does NOT have contain "bad" code. It might just be code that looks bad. That\'s why it is very important to go through the files manually and have a look at the suspicious lines of code. If you are certain they are ok - whitelist the file.<br>
    Many popular plugins\' and themes\' files can appear on the scan list. Again this does not mean they will do harm to your site. It just means you need to have a closer look at their content.</p>';

    echo '</div>';


    echo '<div id="sn_ms_results">';
    if ($last_run) {
      $results = self::run_scan('get_results');
      echo $results;
    }
    echo '</div>';

    if ($last_run) {
      echo '<p class="center"><br><br><a href="#" id="sn-ms-reset-whitelist" class="button button-secondary input-button red">Reset whitelisted files list</a></p>';
    }
  } // malware_page


  // display warning if test were never run
  static function warnings() {
    $last_run = self::get_results('last_run');

    if (!wf_sn::is_plugin_page() || !wf_sn::is_activated()) {
      return;
    }
    
    $memory = ini_get('memory_limit');
    if (wf_sn::is_plugin_page() && (int) $memory != -1 && (int) $memory < 64 && stripos($memory, 'G') === false) {
      echo '<div class="notice updated sn_ms_warning"><p>Your PHP memory limit is only ' . $memory . 'MB. Malware Scanner may not be able to scan all your files. Please ask your hoster to increse the limit.</p></div>';
    }

    if (!empty($last_run) && wf_sn::is_plugin_page() && (current_time('timestamp') - 30*24*60*60) > $last_run) {
      echo '<div class="notice notice-error wf-sn-ms-not-run"><p>Security Ninja Malware Scanner <strong>tests were not run for more than 30 days.</strong> It\'s advisable to run them once in a while. Click "Scan files" to run them now and check your files for malware.</p></div>';
    }
  } // run_tests_warning


  // clean-up when deactivated
  static function deactivate() {
    delete_option(WF_SN_MS_OPTIONS);
    delete_option(WF_SN_MS_RESULTS);
    delete_option(WF_SN_MS_CACHE);
    delete_option(WF_SN_MS_WHITELIST);
    delete_option(WF_SN_MS_DELETELIST);
  } // deactivate
} // wf_sn_ms class


// hook everything up
add_action('plugins_loaded', array('wf_sn_ms', 'init'));
// when deativated clean up
register_deactivation_hook(WF_SN_BASE_FILE, array('wf_sn_ms', 'deactivate'));
